# Path: scaffold/artisans/distill/core/assembler/content/facade.py

"""
=================================================================================
== THE CONTENT WEAVER FACADE (V-Ω-ETERNAL-APOTHEOSIS-ULTIMA)                   ==
=================================================================================
LIF: INFINITY

The High Priest of the Weaving Sanctum. This artisan is a pure Conductor. Its
one true purpose is to receive the plea from the BudgetWeaver, forge the sacred
Gnostic Context, and command the WeavingOrchestrator to begin the Great Work.
=================================================================================
"""
from pathlib import Path
from typing import Set, Optional, Dict, List, Any

from .contracts import WeavingContext
from .orchestrator import WeavingOrchestrator
from ......core.cortex.contracts import DistillationProfile
from ...skeletonizer import GnosticSkeletonizer
from ......core.cortex.contracts import FileGnosis


class ContentWeaver:
    """The Sovereign Facade for the Content Weaving pipeline."""

    def __init__(
            self,
            profile: DistillationProfile,
            skeletonizer: GnosticSkeletonizer,
            heat_map: Optional[Dict[str, Set[int]]] = None,
            runtime_values: Optional[Dict[str, Dict[int, List[str]]]] = None,
            perf_stats: Optional[Dict[str, Dict[str, Any]]] = None
    ):
        # The Facade is born with the tools it needs to forge the context.
        self.profile = profile
        self.skeletonizer = skeletonizer
        self.heat_map = heat_map or {}
        self.runtime_values = runtime_values or {}
        self.perf_stats = perf_stats or {}
        self.slicer = getattr(profile, 'slicer', None)
        self.diff_context = getattr(profile, 'diff_context', False)

        # The Orchestrator is summoned.
        self.orchestrator = WeavingOrchestrator()

    def weave(self, gnosis: FileGnosis, project_root: Path, active_symbols: Optional[Set[str]]) -> str:
        """The Grand Rite of Weaving."""
        context = self._forge_context(gnosis, project_root, active_symbols)
        return self.orchestrator.weave(context)

    def weave_with_annotations(
            self,
            gnosis: FileGnosis,
            project_root: Path,
            active_symbols: Set[str],
            heat_lines: Set[int],
            runtime_notes: Dict[int, List[str]]
    ) -> str:
        """The Special Rite of Annotation."""
        # We must update the context maps for this specific rite.
        path_str = str(gnosis.path).replace('\\', '/')
        self.heat_map[path_str] = heat_lines
        self.runtime_values[path_str] = runtime_notes

        context = self._forge_context(gnosis, project_root, active_symbols)
        # Weave with the updated context.
        return self.orchestrator.weave(context)

    def _forge_context(self, gnosis: FileGnosis, project_root: Path,
                       active_symbols: Optional[Set[str]]) -> WeavingContext:
        """Forges the immutable vessel of Gnostic Context."""
        return WeavingContext(
            profile=self.profile,
            gnosis=gnosis,
            project_root=project_root,
            active_symbols=active_symbols,
            heat_map=self.heat_map,
            runtime_values=self.runtime_values,
            perf_stats=self.perf_stats,
            skeletonizer=self.skeletonizer,
            slicer=self.slicer,
            diff_context=self.diff_context
        )


# Path: artisans/fortress/artisan.py
# ----------------------------------

import shutil
import subprocess
from rich.table import Table

from ...core.artisan import BaseArtisan
from ...interfaces.base import ScaffoldResult
from ...interfaces.requests import FortressRequest
from ...help_registry import register_artisan
from ...contracts.heresy_contracts import ArtisanHeresy


@register_artisan("fortify")
class FortressArtisan(BaseArtisan[FortressRequest]):
    """
    =================================================================================
    == THE FORTRESS (V-Ω-ACTIVE-DEFENSE)                                           ==
    =================================================================================
    LIF: 10,000,000,000

    Audits dependencies, scans for secrets, and static analysis vulnerabilities.
    """

    def execute(self, request: FortressRequest) -> ScaffoldResult:
        self.logger.info("The Fortress raises its shields...")

        vulnerabilities = []

        # 1. Dependency Audit (Python)
        if (self.project_root / "poetry.lock").exists() or (self.project_root / "requirements.txt").exists():
            if shutil.which("safety"):
                self.logger.info("Scanning Python dependencies with 'safety'...")
                # Mocking safety output parsing for V1
                # res = subprocess.run(["safety", "check", "--json"], ...)
                pass
            else:
                self.logger.warn("'safety' tool missing. Skipping Python audit.")

        # 2. Dependency Audit (Node)
        if (self.project_root / "package.json").exists():
            self.logger.info("Scanning Node dependencies with 'npm audit'...")
            try:
                res = subprocess.run(["npm", "audit", "--json"], cwd=self.project_root, capture_output=True, text=True)
                # Parse JSON
                pass
            except Exception:
                pass

        # 3. Static Analysis (Bandit for Python)
        if shutil.which("bandit"):
            self.logger.info("Scanning code with 'bandit'...")
            try:
                # bandit -r . -f json
                res = subprocess.run(["bandit", "-r", ".", "-f", "json"], cwd=self.project_root, capture_output=True,
                                     text=True)
                import json
                report = json.loads(res.stdout)
                if report.get("results"):
                    for issue in report["results"]:
                        vulnerabilities.append({
                            "severity": issue["issue_severity"],
                            "file": issue["filename"],
                            "line": issue["line_number"],
                            "description": issue["issue_text"]
                        })
            except Exception:
                pass

        # 4. Proclaim
        if not vulnerabilities:
            return self.success("The Fortress is secure. No obvious cracks found.")

        table = Table(title="[bold red]Security Breaches Detected[/bold red]", border_style="red")
        table.add_column("Severity", style="bold")
        table.add_column("Location", style="cyan")
        table.add_column("Threat")

        for v in vulnerabilities:
            color = "red" if v['severity'] == 'HIGH' else "yellow"
            table.add_row(f"[{color}]{v['severity']}[/{color}]", f"{v['file']}:{v['line']}", v['description'])

        self.console.print(table)

        return self.failure(f"Found {len(vulnerabilities)} vulnerabilities.")


# === [scaffold/core/cortex/data_flow.py] - SECTION 1 of 1: The Taint Tracker ===
import re
from typing import List, Dict, Set, Optional
from pathlib import Path
from .contracts import CortexMemory
from ...logger import Scribe

Logger = Scribe("DataFlowEngine")


class DataFlowEngine:
    """
    =============================================================================
    == THE RIVER OF DATA (V-Ω-SYMBOLIC-TAINT-TRACKER)                          ==
    =============================================================================
    LIF: 10,000,000,000

    Traces the flow of 'Tainted Symbols' (variables of interest) across the
    codebase. It uses the Gnostic Cortex's symbol map to jump from file to file.

    Logic:
    1. Find definition of Symbol S.
    2. Find all files that Import S.
    3. Find all files that use S string-wise (heuristic resonance).
    """

    def __init__(self, memory: CortexMemory):
        self.memory = memory

    def trace(self, symbols: List[str]) -> Dict[str, int]:
        """
        Returns a map of {file_path: score_boost}.
        """
        impact_map: Dict[str, int] = {}

        for symbol in symbols:
            Logger.info(f"Tracing the flow of data: [cyan]'{symbol}'[/cyan]...")

            # 1. Direct Definition (Where is it born?)
            # symbol_multimap maps "User" -> ["src/models/user.py"]
            origins = self.memory.symbol_multimap.get(symbol, [])

            # Also check for qualified names e.g. "auth.SECRET_KEY"
            if not origins and "." in symbol:
                simple_name = symbol.split(".")[-1]
                origins = self.memory.symbol_multimap.get(simple_name, [])

            if not origins:
                Logger.verbose(f"   -> Symbol '{symbol}' origin not found in Cortex.")

            # 2. Causality Expansion (Where does it go?)
            tainted_files = set(origins)

            # Scan inventory for usage
            # This is a heuristic scan. A full DFG requires deep AST analysis of every file.
            # We use the "Resonance Judge" logic here: text search in AST-aware context.

            for gnosis in self.memory.inventory:
                path_str = str(gnosis.path).replace('\\', '/')

                # Skip if already tainted (origin)
                if path_str in tainted_files:
                    impact_map[path_str] = impact_map.get(path_str, 0) + 100  # High boost for origin
                    continue

                # Check Imports (Strong Link)
                # If file imports the symbol explicitly
                if symbol in gnosis.imported_symbols:
                    impact_map[path_str] = impact_map.get(path_str, 0) + 50
                    tainted_files.add(path_str)
                    continue

                # Check Content Resonance (Weak Link)
                # Does the file contain the string?
                # We use the pre-computed semantic_tags or check content if available (slow)
                # Fast check: symbol name in AST metrics
                is_used = False
                if gnosis.ast_metrics:
                    # Check function arguments for data passing
                    for func in gnosis.ast_metrics.get('functions', []):
                        if symbol in func.get('args_list', []):
                            is_used = True
                            break

                if is_used:
                    impact_map[path_str] = impact_map.get(path_str, 0) + 25
                    tainted_files.add(path_str)

        Logger.success(f"Data Flow Analysis complete. {len(impact_map)} files touched by the river.")
        return impact_map




# Path: scaffold/symphony/renderers/cinematic_renderer/layout_engine.py
# ---------------------------------------------------------------------

from rich.layout import Layout
from .widgets import (
    HeaderWidget, FooterWidget, TimelineWidget,
    OutputWidget, ContextWidget, ResourceMonitorWidget
)
from .state import CinematicState


class GnosticLayoutEngine:
    """
    =================================================================================
    == THE ARCHITECT OF SPACE (V-Ω-ADAPTIVE-GRID)                                  ==
    =================================================================================
    Constructs the renderable view from the state.
    """

    def __init__(self):
        self.layout = Layout()
        self.layout.split(
            Layout(name="header", size=3),
            Layout(name="main", ratio=1),
            Layout(name="footer", size=1)
        )
        self.layout["main"].split_row(
            Layout(name="sidebar", ratio=1),
            Layout(name="body", ratio=3)
        )
        self.layout["sidebar"].split(
            Layout(name="timeline", ratio=2),
            Layout(name="vitals", size=6),
            Layout(name="context", size=8)
        )

        # Instantiate Widgets
        self.header = HeaderWidget()
        self.footer = FooterWidget()
        self.timeline = TimelineWidget()
        self.monitor = ResourceMonitorWidget()
        self.context = ContextWidget()
        self.output = OutputWidget()

    def render(self, state: CinematicState) -> Layout:
        """Composes the current frame."""
        self.layout["header"].update(self.header.render(state))
        self.layout["footer"].update(self.footer.render(state))

        self.layout["timeline"].update(self.timeline.render(state))
        self.layout["vitals"].update(self.monitor.render(state))
        self.layout["context"].update(self.context.render(state))

        self.layout["body"].update(self.output.render(state))

        return self.layout


# Path: artisans/codex/artisan.py
# -------------------------------

import re
import os
import subprocess
from pathlib import Path
from typing import List, Dict

from rich.panel import Panel
from rich.markdown import Markdown

from ...core.artisan import BaseArtisan
from ...interfaces.base import ScaffoldResult, Artifact
from ...interfaces.requests import CodexRequest
from ...help_registry import register_artisan
from ...contracts.heresy_contracts import ArtisanHeresy
from ...core.cortex.engine import GnosticCortex
from ...utils import atomic_write


@register_artisan("codex")
class CodexArtisan(BaseArtisan[CodexRequest]):
    """
    =================================================================================
    == THE LIVING WIKI (V-Ω-EXECUTABLE-DOCS)                                       ==
    =================================================================================
    LIF: 10,000,000,000

    Generates documentation that compiles and tests itself.
    It ensures the Map (Docs) never drifts from the Territory (Code).
    """

    VERIFY_BLOCK_REGEX = re.compile(r'<!--\s*@scaffold-verify\s*-->\s*```(\w+)\n(.*?)```', re.DOTALL)

    def execute(self, request: CodexRequest) -> ScaffoldResult:
        if request.codex_command == "build":
            return self._conduct_build_rite(request)
        elif request.codex_command == "verify":
            return self._conduct_verify_rite(request)
        elif request.codex_command == "cartography":
            return self._conduct_cartography_rite(request)

        return self.failure("Unknown Codex Rite.")

    def _conduct_build_rite(self, request: CodexRequest) -> ScaffoldResult:
        """Forges the static site."""
        target = self.project_root / request.output_dir
        self.logger.info(f"Forging the Codex at [cyan]{target}[/cyan]...")

        # Check for MkDocs
        if not (self.project_root / "mkdocs.yml").exists():
            self._forge_default_mkdocs()

        try:
            subprocess.run(["mkdocs", "build", "-d", str(target)], cwd=self.project_root, check=True)
            if request.serve:
                self.logger.info("Serving Codex...")
                subprocess.run(["mkdocs", "serve"], cwd=self.project_root)
            return self.success(f"Codex forged at {target}")
        except FileNotFoundError:
            raise ArtisanHeresy("The 'mkdocs' artisan is missing. pip install mkdocs")
        except subprocess.CalledProcessError as e:
            raise ArtisanHeresy(f"The Codex build faltered: {e}")

    def _conduct_verify_rite(self, request: CodexRequest) -> ScaffoldResult:
        """
        Scans Markdown files for executable blocks and tests them.
        """
        self.logger.info("The Codex Inquisitor awakens to verify truth in documentation...")

        docs_dir = self.project_root / "docs"  # Default
        if not docs_dir.exists(): docs_dir = self.project_root

        markdown_files = list(docs_dir.rglob("*.md"))
        failures = []
        verified_count = 0

        for md_file in markdown_files:
            content = md_file.read_text(encoding='utf-8')
            matches = self.VERIFY_BLOCK_REGEX.findall(content)

            for lang, code in matches:
                self.logger.verbose(f"Verifying block in {md_file.name} ({lang})...")
                if not self._execute_snippet(lang, code):
                    failures.append(f"{md_file.name}: Failed to execute {lang} block.")
                verified_count += 1

        if failures:
            return self.failure(f"Verification Failed. {len(failures)} heresies found.", data={"failures": failures})

        return self.success(f"Codex Verified. {verified_count} code blocks are pure.")

    def _execute_snippet(self, lang: str, code: str) -> bool:
        """Executes a snippet in a safe-ish subprocess."""
        try:
            if lang == "python":
                subprocess.run([sys.executable, "-c", code], check=True, capture_output=True)
            elif lang in ["bash", "sh"]:
                subprocess.run(code, shell=True, check=True, capture_output=True)
            else:
                self.logger.warn(f"Cannot verify language: {lang}")
                return True  # Pass for unknown langs
            return True
        except Exception:
            return False

    def _conduct_cartography_rite(self, request: CodexRequest) -> ScaffoldResult:
        """Generates architectural diagrams."""
        self.logger.info("Summoning the Cartographer...")

        cortex = GnosticCortex(self.project_root)
        memory = cortex.perceive()

        from ...artisans.distill.core.scribes.mermaid_scribe import MermaidScribe
        scribe = MermaidScribe(self.project_root, memory)

        # Determine centrality for top-level diagram
        central_files = [f for f in memory.inventory if f.centrality_score > 20]
        diagram = scribe.inscribe(central_files)

        out_path = self.project_root / "docs" / "architecture.mermaid"
        out_path.parent.mkdir(exist_ok=True)
        atomic_write(out_path, diagram, self.logger, self.project_root)

        return self.success("Cartography complete.", artifacts=[Artifact(path=out_path, type="file", action="created")])

    def _forge_default_mkdocs(self):
        content = f"site_name: {self.project_root.name}\ntheme: material\n"
        (self.project_root / "mkdocs.yml").write_text(content)


# Path: core/lsp/base/features/type_hierarchy/__init__.py
# -------------------------------------------------------
from .engine import TypeHierarchyEngine
from .contracts import TypeHierarchyProvider
from .models import (
    TypeHierarchyItem,
    TypeHierarchyPrepareParams,
    TypeHierarchySupertypesParams,
    TypeHierarchySubtypesParams,
    TypeHierarchyOptions
)

__all__ = [
    "TypeHierarchyEngine", "TypeHierarchyProvider",
    "TypeHierarchyItem",
    "TypeHierarchyPrepareParams", "TypeHierarchySupertypesParams", "TypeHierarchySubtypesParams",
    "TypeHierarchyOptions"
]


# Path: core/daemon/serializer/registry.py
# ----------------------------------------
# LIF: INFINITY | ROLE: ENCODER_ROUTER
from typing import Any, Callable, List, Tuple, Type


class EncoderRegistry:
    """
    [THE CODEX]
    Maintains the mapping between Python Types and their Transmutation Rites.
    """

    def __init__(self):
        # List of (Type, EncoderFunction)
        # We use a list to support inheritance checks (isinstance)
        self._encoders: List[Tuple[Type, Callable[[Any], Any]]] = []

    def register(self, type_class: Type, encoder: Callable[[Any], Any]):
        """
        Consecrates a new handler for a specific type.
        """
        self._encoders.append((type_class, encoder))

    def resolve(self, obj: Any) -> Callable[[Any], Any]:
        """
        Divines the correct encoder for the given object.
        Returns None if no matching rite is found.
        """
        for type_class, encoder in self._encoders:
            if isinstance(obj, type_class):
                return encoder
        return None




# Path: core/lsp/scaffold_features/workspace/watcher.py
# -----------------------------------------------------
from typing import List
from ...base.features.workspace.models import FileEvent, FileChangeType
from ....base.utils.uri import UriUtils


class ScaffoldFluxProvider:
    """
    [THE CAUSAL EYE]
    Reacts to changes in specific Scaffold files to keep the Cortex warm.
    """

    def __init__(self, server: Any):
        self.server = server

    def on_file_events(self, events: List[FileEvent]):
        for event in events:
            path = UriUtils.to_fs_path(event.uri)

            # [ASCENSION 1]: BLUEPRINT SENSITIVITY
            if path.suffix in ('.scaffold', '.arch', '.splane'):
                if event.type == FileChangeType.Changed:
                    # Trigger a re-analysis if the law has changed
                    self.server.diagnostics.schedule_validation(event.uri, 0, priority=True)

            # [ASCENSION 5]: LOCKFILE RESONANCE
            if path.name == "scaffold.lock":
                # Reality has shifted externally (e.g. Git pull)
                # Proclaim to the UI that a "Global Survey" is recommended
                pass


# Path: artisans/schema/engine.py
# -------------------------------

import shutil
import subprocess
from pathlib import Path
from typing import Tuple, Optional

from ...contracts.heresy_contracts import ArtisanHeresy
from ...logger import Scribe

Logger = Scribe("SchemaEngineCore")


class SchemaEngine:
    """
    The Polyglot Adapter for ORMs.
    Currently supports: Alembic (Python), Prisma (Node/Python).
    """

    def __init__(self, root: Path):
        self.root = root

    def divine_stack(self) -> str:
        """Determines which ORM is governing the sanctum."""
        if (self.root / "alembic.ini").exists():
            return "alembic"
        if (self.root / "prisma" / "schema.prisma").exists() or (self.root / "schema.prisma").exists():
            return "prisma"
        return "unknown"

    def check_drift(self, stack: str) -> Tuple[bool, str]:
        """
        Returns (has_drift, details).
        """
        if stack == "alembic":
            # Alembic check returns non-zero if drift?
            # Usually `alembic check` exists in newer versions.
            try:
                res = subprocess.run(
                    ["alembic", "check"],
                    cwd=self.root, capture_output=True, text=True
                )
                if res.returncode != 0:
                    return True, res.stdout or res.stderr
                return False, "Models match Migration History."
            except FileNotFoundError:
                # Fallback check
                return False, "Alembic check command not found (old version?)"

        elif stack == "prisma":
            # Prisma migrate status
            # Exit code 1 usually means drift or unapplied migrations
            res = subprocess.run(
                ["npx", "prisma", "migrate", "status"],
                cwd=self.root, capture_output=True, text=True
            )
            # Prisma output needs parsing.
            if "Following migration have not yet been applied" in res.stdout:
                return True, "Pending migrations detected."
            if "Database schema is not in sync" in res.stdout:
                return True, "Schema drift detected."
            return False, "Sync."

        return False, "Unknown stack."

    def generate_migration(self, stack: str, message: str) -> Optional[Path]:
        """
        Generates the migration file.
        """
        Logger.info(f"Forging migration for {stack}: '{message}'")

        if stack == "alembic":
            try:
                subprocess.run(
                    ["alembic", "revision", "--autogenerate", "-m", message],
                    cwd=self.root, check=True
                )
                # Find the new file (heuristic: newest in versions)
                versions = list((self.root / "alembic" / "versions").glob("*.py"))
                if versions:
                    return max(versions, key=lambda f: f.stat().st_mtime)
            except subprocess.CalledProcessError:
                raise ArtisanHeresy("Alembic failed to autogenerate. Check your model imports in env.py.")

        elif stack == "prisma":
            # Prisma `migrate dev --create-only` generates sql without applying
            try:
                subprocess.run(
                    ["npx", "prisma", "migrate", "dev", "--create-only", "--name", message],
                    cwd=self.root, check=True
                )
                # Prisma puts it in prisma/migrations/<timestamp>_name/migration.sql
                # We return the dir for now
                migrations_dir = self.root / "prisma" / "migrations"
                if migrations_dir.exists():
                    # Find newest dir
                    subdirs = [d for d in migrations_dir.iterdir() if d.is_dir()]
                    if subdirs:
                        newest = max(subdirs, key=lambda d: d.stat().st_mtime)
                        return newest / "migration.sql"
            except subprocess.CalledProcessError:
                raise ArtisanHeresy("Prisma generation failed.")

        return None

    def apply_migration(self, stack: str) -> Tuple[bool, str]:
        """
        Applies pending migrations.
        """
        Logger.info(f"Applying mutations via {stack}...")

        cmd = []
        if stack == "alembic":
            cmd = ["alembic", "upgrade", "head"]
        elif stack == "prisma":
            cmd = ["npx", "prisma", "migrate", "deploy"]

        try:
            res = subprocess.run(cmd, cwd=self.root, capture_output=True, text=True, check=True)
            return True, res.stdout
        except subprocess.CalledProcessError as e:
            return False, e.stderr + e.stdout


# Gnostic Codex: scaffold/artisans/template_engine/engine/alias_oracle.py
# ---------------------------------------------------------------------
# LIF: ∞ (THE MASTER LINGUIST)
#
# This divine artisan is the one true Master Linguist of the Forge. It has been
# forged from the soul of the old `_load_aliases` rite into a sovereign,
# intelligent, and hyper-aware Oracle of Aliases.
#
# ### THE PANTHEON OF 12 GAME-CHANGING, MIND-BLOWING ELEVATIONS:
#
# 1.  **The Sovereign Soul:** It is now a pure, self-contained class, its purpose absolute.
# 2.  **The Gnostic Triage of Realms:** It understands the hierarchy of Gnosis:
#     Project Config > Global Config > System Defaults.
# 3.  **The Unbreakable Ward of Paradox:** Its Gaze is shielded. A profane or
#     corrupted `config.json` will not shatter the engine.
# 4.  **The Luminous Voice:** Its every Gaze is chronicled with rich, Gnostic detail.
# 5.  **The Reverse Gaze:** Possesses a new, divine `reverse_lookup` rite, allowing
#     it to prophesy that `template.py` is the righteous scripture for `__init__.py`.
# 6.  **The Pure Gnostic Contract:** Its `load` and `get` rites are unbreakable contracts.
# 7.  **The Rite of Purging:** Its `purge` rite returns its memory to the void.
# 8.  **The Reusable Gaze (`_load_from_path`):** Its core Gaze is a pure, reusable
#     artisan, ready for future expansion to other configuration sources.
# 9.  **The Unbreakable Anchor:** It correctly anchors its Gaze to the `project_root`,
#     annihilating heresies of relativity.
# 10. **The Gnostic Grimoire:** Its `DEFAULT_ALIASES` are now its sacred, internal
#     grimoire, the foundation of its wisdom.
# 11. **The Polyglot Mind:** The entire system is forged to be extensible, ready to
#     learn new aliases from any source.
# 12. **The Final Word:** It is the one true, definitive, and eternal artisan for all
#     matters of template alias resolution.

import json
from pathlib import Path
from typing import Dict, Optional

from ....logger import Scribe

Logger = Scribe("AliasOracle")

class AliasOracle:
    """The Master Linguist. Manages mappings between filenames and template extensions."""

    DEFAULT_ALIASES: Dict[str, str] = {
        "__init__.py": "py",
        "mod.rs": "rs",
        "index.ts": "ts",
        "index.js": "js",
        "index.tsx": "tsx",
        "main.go": "go",
        "main.py": "py",
        "Cargo.toml": "rust",
        "pyproject.toml": "python",
        "package.json": "node",
        "go.mod": "go",
    }

    def __init__(self, project_root: Optional[Path]):
        self.project_root = project_root
        self.aliases: Dict[str, str] = {}
        self._reverse_map: Dict[str, str] = {}

    def load(self):
        """Performs the Gnostic Triage to load aliases from all realms."""
        Logger.verbose("The Alias Oracle awakens its Gaze...")
        # Tier 3: System Defaults (The Foundation)
        final_aliases = self.DEFAULT_ALIASES.copy()

        # Tier 2: Global Config (The Architect's General Will)
        global_config = Path.home() / ".scaffold" / "config.json"
        global_aliases = self._load_from_path(global_config)
        final_aliases.update(global_aliases)

        # Tier 1: Project Config (The Architect's Specific Will)
        if self.project_root:
            project_config = self.project_root / ".scaffold" / "config.json"
            project_aliases = self._load_from_path(project_config)
            final_aliases.update(project_aliases)

        self.aliases = final_aliases
        self._reverse_map = {v: k for k, v in self.aliases.items()}
        Logger.verbose(f"Alias Gaze complete. {len(self.aliases)} total aliases are known.")

    def _load_from_path(self, path: Path) -> Dict[str, str]:
        """A pure, reusable Gaze upon a single config scripture."""
        if path.is_file():
            try:
                data = json.loads(path.read_text(encoding='utf-8'))
                user_aliases = data.get("template_aliases", {})
                if user_aliases:
                    Logger.verbose(f"Perceived {len(user_aliases)} custom aliases from '{path.name}'.")
                    return user_aliases
            except (json.JSONDecodeError, IOError):
                Logger.warn(f"A paradox was perceived in the Polyglot scripture at '{path}'.")
        return {}

    def get(self, filename: str) -> Optional[str]:
        """Performs a Gaze for a filename's aliased soul."""
        return self.aliases.get(filename)

    def reverse_lookup(self, template_ext: str) -> Optional[str]:
        """[THE REVERSE GAZE] Finds the filename that maps to a template extension."""
        return self._reverse_map.get(template_ext)

    def purge(self):
        """The Rite of Purging."""
        self.aliases.clear()
        self._reverse_map.clear()
        Logger.verbose("The Alias Oracle's memory has been returned to the void.")


# Path: core/semantics/__init__.py
# --------------------------------

"""
=================================================================================
== THE SEMANTIC CORTEX (V-Ω-CORE-FACULTY)                                      ==
=================================================================================
This sanctum houses the faculties of Intent Analysis, Semantic Retrieval, and
Relevance Reranking. It is the engine's ability to understand "Meaning" beyond
mere syntax.
"""

from .engine import SemanticEngine
from .contracts import UserIntent, SemanticHit, QueryIntent
from .retriever import SemanticRetriever
from .intent import IntentAnalyzer
from .reranker import RelevanceReranker

__all__ = [
    "SemanticEngine",
    "UserIntent",
    "SemanticHit",
    "QueryIntent",
    "SemanticRetriever",
    "IntentAnalyzer",
    "RelevanceReranker"
]


# Path: scaffold/artisans/translate/artisan.py
# --------------------------------------------

from pathlib import Path
from rich.panel import Panel
from rich.syntax import Syntax

from ...core.artisan import BaseArtisan
from ...interfaces.base import ScaffoldResult
from ...interfaces.requests import TranslateRequest
from ...core.ai.engine import AIEngine
from ...help_registry import register_artisan
from ...inquisitor import get_treesitter_gnosis


@register_artisan("translate")
class TranslateArtisan(BaseArtisan[TranslateRequest]):
    """
    =============================================================================
    == THE ROSETTA STONE (V-Ω-ARCHITECTURAL-TRANSLATOR)                        ==
    =============================================================================
    Transmutes code between tongues while preserving the Gnostic Soul.
    Uses AST analysis to prime the AI with structural context.
    """

    def execute(self, request: TranslateRequest) -> ScaffoldResult:
        source_path = (self.project_root / request.source_path).resolve()
        if not source_path.exists():
            return self.failure("Source scripture not found.")

        self.logger.info(f"Translating {source_path.name} to {request.target_lang}...")

        # 1. Extract Gnosis (AST)
        content = source_path.read_text(encoding='utf-8')
        dossier = get_treesitter_gnosis(source_path, content)

        # Summarize Structure for the AI
        structure_summary = []
        if "functions" in dossier:
            structure_summary.append(f"Functions: {', '.join([f['name'] for f in dossier['functions']])}")
        if "classes" in dossier:
            structure_summary.append(f"Classes: {', '.join([c['name'] for c in dossier['classes']])}")

        context_str = "\n".join(structure_summary)

        # 2. The Prompt
        prompt = (
            f"Translate the following code from {request.source_lang} to {request.target_lang}.\n"
            f"Context: {context_str}\n"
            "Maintain the exact architectural pattern (Class/Function structure). "
            "Use idiomatic patterns for the target language (e.g. Structs/Impl for Rust, Classes for Python).\n\n"
            f"Code:\n```\n{content}\n```"
        )

        # 3. The Communion
        ai = AIEngine.get_instance()
        translated_code = ai.ignite(prompt, model="smart")

        # 4. The Proclamation
        self.console.print(Panel(
            Syntax(translated_code, request.target_lang, theme="monokai"),
            title=f"[bold green]Translation: {request.target_lang.upper()}[/bold green]"
        ))

        return self.success("Translation complete.", data={"content": translated_code})


# scaffold/core/simulation/conductor/reality_forge.py

import json
import os
import shutil
import subprocess
from pathlib import Path
from typing import Any, List

from ....core.cortex.knowledge import KnowledgeBase
from ....core.kernel.chronicle import update_chronicle
from ....contracts.data_contracts import GnosticWriteResult, InscriptionAction
from ....utils import hash_file
from ....logger import Scribe

try:
    import pathspec

    PATHSPEC_AVAILABLE = True
except ImportError:
    PATHSPEC_AVAILABLE = False

Logger = Scribe("RealityForge")


class RealityForge:
    """
    =================================================================================
    == THE REALITY FORGE (V-Ω-STATE-AWARE)                                         ==
    =================================================================================
    Constructs ephemeral realities and chronicles their initial state.
    """

    def __init__(self, real_root: Path, sim_root: Path):
        self.real_root = real_root
        self.sim_root = sim_root
        self.ignore_spec = self._forge_ignore_spec()
        self.manifest_path = self.sim_root / ".scaffold" / "sim_manifest.json"

    def materialize_void(self):
        """Forges a clean, empty universe."""
        self._prepare_sanctum()
        self._record_manifest([])  # Void state
        Logger.debug(f"Void materialized at {self.sim_root.name}")

    def materialize_mirror(self):
        """Forges a perfect reflection and records the manifest."""
        self._prepare_sanctum()
        Logger.debug(f"Mirroring reality to {self.sim_root.name}...")

        mirrored_files = []

        # Strategy A: The Git Bridge
        if (self.real_root / ".git").exists() and shutil.which("git"):
            try:
                res = subprocess.run(
                    ["git", "ls-files", "-c", "-o", "--exclude-standard"],
                    cwd=self.real_root, capture_output=True, text=True, timeout=10
                )
                if res.returncode == 0:
                    paths = [Path(p) for p in res.stdout.splitlines() if p.strip()]
                    if paths:
                        for rel in paths:
                            if self._copy_file(rel):
                                mirrored_files.append(str(rel.as_posix()))

                        self._ensure_internal_structures()
                        self._record_manifest(mirrored_files)
                        self._check_lockfile()
                        return
            except Exception as e:
                Logger.warn(f"Git Bridge faltered ({e}). Falling back to manual copy.")

        # Strategy B: The Manual Walk
        files_to_copy = self._scan_reality(self.real_root)
        for src in files_to_copy:
            try:
                rel = src.relative_to(self.real_root)
                if self._copy_file(rel):
                    mirrored_files.append(str(rel.as_posix()))
            except Exception:
                pass

        self._ensure_internal_structures()
        self._record_manifest(mirrored_files)
        self._check_lockfile()

    def _copy_file(self, rel_path: Path) -> bool:
        """Atomic copy helper. Returns True if successful."""
        src = self.real_root / rel_path
        dst = self.sim_root / rel_path
        try:
            if src.exists() and src.is_file():
                dst.parent.mkdir(parents=True, exist_ok=True)
                shutil.copy2(src, dst)
                return True
        except Exception:
            pass
        return False

    def _record_manifest(self, files: List[str]):
        """
        [THE RITE OF RECORD]
        Writes the list of mirrored files to disk. This is the Absolute Truth
        of what exists in the simulation at T=0.
        """
        data = {"files": sorted(files)}
        self.manifest_path.write_text(json.dumps(data, indent=2), encoding='utf-8')

    def annihilate(self):
        try:
            shutil.rmtree(self.sim_root, ignore_errors=True)
        except Exception:
            pass

    def _prepare_sanctum(self):
        if self.sim_root.exists():
            shutil.rmtree(self.sim_root, ignore_errors=True)
        self.sim_root.mkdir(parents=True, exist_ok=True)
        self._ensure_internal_structures()

    def _ensure_internal_structures(self):
        (self.sim_root / ".scaffold").mkdir(exist_ok=True)

    def _check_lockfile(self):
        if not (self.real_root / "scaffold.lock").exists():
            self._conduct_ghost_adoption()

    def _conduct_ghost_adoption(self):
        write_dossier = []
        # We scan the SIMULATION to adopt what we actually copied
        sim_files = self._scan_reality(self.sim_root)
        for path in sim_files:
            write_dossier.append(GnosticWriteResult(
                success=True, path=path, action_taken=InscriptionAction.ADOPTED,
                bytes_written=path.stat().st_size, gnostic_fingerprint=hash_file(path), blueprint_origin=None
            ))
        update_chronicle(
            project_root=self.sim_root, blueprint_path=Path("ghost.scaffold"),
            rite_dossier={}, old_lock_data={}, write_dossier=write_dossier,
            final_vars={}, rite_name="Ghost Adoption"
        )

    # ... [Include _forge_ignore_spec, _is_ignored, _scan_reality from previous] ...
    def _forge_ignore_spec(self) -> Any:
        patterns = set()
        patterns.update(KnowledgeBase.ABYSS_DIRECTORIES)
        patterns.update(KnowledgeBase.ABYSS_GLOBS)
        patterns.add(".scaffold")
        patterns.add("simulation")
        gitignore = self.real_root / ".gitignore"
        if gitignore.exists():
            try:
                with open(gitignore, 'r', encoding='utf-8') as f:
                    for line in f:
                        line = line.strip()
                        if line and not line.startswith('#'):
                            patterns.add(line)
            except Exception:
                pass
        if PATHSPEC_AVAILABLE:
            return pathspec.PathSpec.from_lines('gitwildmatch', patterns)
        return patterns

    def _is_ignored(self, rel_path: str) -> bool:
        if PATHSPEC_AVAILABLE: return self.ignore_spec.match_file(rel_path)
        return False

    def _scan_reality(self, root: Path) -> List[Path]:
        valid_files = []
        for dirpath, dirnames, filenames in os.walk(root):
            for i in range(len(dirnames) - 1, -1, -1):
                d = dirnames[i]
                full_dir = Path(dirpath) / d
                try:
                    rel_dir = str(full_dir.relative_to(root)).replace("\\", "/")
                except:
                    continue
                if d == ".git" or d == ".scaffold" or self._is_ignored(rel_dir) or self._is_ignored(rel_dir + "/"):
                    del dirnames[i]
            for f in filenames:
                full_path = Path(dirpath) / f
                try:
                    rel_path = str(full_path.relative_to(root)).replace("\\", "/")
                except:
                    continue
                if not self._is_ignored(rel_path): valid_files.append(full_path)
        return valid_files


# scaffold/semantic_injection/directives/ui_knowledge/charts.py

"""
=================================================================================
== THE ORACLE OF VISUALIZATION (V-Ω-CHARTS)                                    ==
=================================================================================
This artisan generates responsive, theme-aware chart wrappers.
It assumes the presence of 'recharts' in the project, the industry standard.
"""
from textwrap import dedent
from typing import List, Tuple

from ..registry import ComponentRegistry


@ComponentRegistry.register("line-chart")
def forge_line_chart(name: str, props: List[Tuple[str, str]]) -> str:
    """
    Forges a responsive Line Chart component.
    """
    return dedent(f"""
        import React from 'react';
        import {{ Line, LineChart as ReLineChart, ResponsiveContainer, Tooltip, XAxis, YAxis, CartesianGrid }} from 'recharts';
        import {{ cn }} from '@/lib/utils';

        export interface {name}Props extends React.HTMLAttributes<HTMLDivElement> {{
          data: any[];
          xKey?: string;
          yKey?: string;
          color?: string;
        }}

        export function {name}({{ 
          className, 
          data, 
          xKey = "name", 
          yKey = "value", 
          color = "hsl(var(--primary))",
          ...props 
        }}: {name}Props) {{
          return (
            <div className={{cn("h-[350px] w-full", className)}} {{...props}}>
              <ResponsiveContainer width="100%" height="100%">
                <ReLineChart data={{data}}>
                  <CartesianGrid strokeDasharray="3 3" className="stroke-muted" />
                  <XAxis 
                    dataKey={{xKey}} 
                    stroke="#888888" 
                    fontSize={{12}} 
                    tickLine={{false}} 
                    axisLine={{false}} 
                  />
                  <YAxis 
                    stroke="#888888" 
                    fontSize={{12}} 
                    tickLine={{false}} 
                    axisLine={{false}} 
                    tickFormatter={{(value) => `$${{value}}`}} 
                  />
                  <Tooltip 
                    contentStyle={{{{ backgroundColor: 'hsl(var(--popover))', borderColor: 'hsl(var(--border))' }}}}
                    itemStyle={{{{ color: 'hsl(var(--popover-foreground))' }}}}
                  />
                  <Line 
                    type="monotone" 
                    dataKey={{yKey}} 
                    stroke={{color}} 
                    strokeWidth={{2}} 
                    dot={{false}}
                    activeDot={{{{ r: 8 }}}}
                  />
                </ReLineChart>
              </ResponsiveContainer>
            </div>
          );
        }}
    """).strip()

@ComponentRegistry.register("bar-chart")
def forge_bar_chart(name: str, props: List[Tuple[str, str]]) -> str:
    """
    Forges a responsive Bar Chart component.
    """
    return dedent(f"""
        import React from 'react';
        import {{ Bar, BarChart as ReBarChart, ResponsiveContainer, Tooltip, XAxis, YAxis }} from 'recharts';
        import {{ cn }} from '@/lib/utils';

        export interface {name}Props extends React.HTMLAttributes<HTMLDivElement> {{
          data: any[];
          xKey?: string;
          yKey?: string;
        }}

        export function {name}({{ 
          className, 
          data, 
          xKey = "name", 
          yKey = "total", 
          ...props 
        }}: {name}Props) {{
          return (
            <div className={{cn("h-[350px] w-full", className)}} {{...props}}>
              <ResponsiveContainer width="100%" height="100%">
                <ReBarChart data={{data}}>
                  <XAxis 
                    dataKey={{xKey}} 
                    stroke="#888888" 
                    fontSize={{12}} 
                    tickLine={{false}} 
                    axisLine={{false}} 
                  />
                  <YAxis 
                    stroke="#888888" 
                    fontSize={{12}} 
                    tickLine={{false}} 
                    axisLine={{false}} 
                    tickFormatter={{(value) => `$${{value}}`}} 
                  />
                  <Tooltip 
                    cursor={{{{ fill: 'hsl(var(--muted))', opacity: 0.3 }}}}
                    contentStyle={{{{ backgroundColor: 'hsl(var(--popover))', borderColor: 'hsl(var(--border))', borderRadius: '6px' }}}}
                  />
                  <Bar 
                    dataKey={{yKey}} 
                    fill="hsl(var(--primary))" 
                    radius={[4, 4, 0, 0]} 
                  />
                </ReBarChart>
              </ResponsiveContainer>
            </div>
          );
        }}
    """).strip()


# Path: scaffold/symphony/conductor_core/resilience/manager.py
# ------------------------------------------------------------

import os
import time
import subprocess
import shlex
import threading
import traceback
from pathlib import Path
from contextlib import contextmanager
from typing import Optional, TYPE_CHECKING, Any, Dict, List, Union, Literal

# --- THE DIVINE SUMMONS OF THE PANTHEON & CONTRACTS ---
from .artifacts import ArtifactKeeper
from .diagnosis import NeuralDiagnostician
from .intercession import IntercessionAltar
from .contracts import IntercessionChoice, IntercessionOutcome, FailureContext
from ....contracts.heresy_contracts import ArtisanHeresy, GuardianHeresy, HeresySeverity
from ....contracts.symphony_contracts import Edict, ActionResult, EdictType
from ....logger import Scribe, get_console

if TYPE_CHECKING:
    from ...conductor.orchestrator import SymphonyConductor
    from ..context import GnosticContextManager

Logger = Scribe('SymphonyResilience')


class SymphonyResilienceManager:
    """
    =================================================================================
    == THE THRONE OF INTERCESSION (V-Ω-RESILIENCE-GOD-ENGINE)                      ==
    =================================================================================
    LIF: 10,000,000,000,000,000

    The Sovereign Guardian of Failure. It manages the lifecycle of paradoxes within
    the Symphony, transforming crashes into opportunities for Gnostic insight and
    interactive redemption.
    """

    def __init__(self, conductor: 'SymphonyConductor'):
        self.conductor = conductor
        # [FACULTY 5] The Forensic Scribe
        self.artifact_keeper = ArtifactKeeper(self.conductor.execution_root)
        # [FACULTY 6] The AI Co-Architect
        self.diagnostician = NeuralDiagnostician()
        self.console = get_console()
        # [FACULTY 9] The Luminous Dossier Forge
        self.altar = IntercessionAltar(self.console)

    @property
    def context(self) -> 'GnosticContextManager':
        """[FACULTY 6] The Telepathic Bridge to the Symphony's living memory."""
        return self.conductor.context_manager

    @contextmanager
    def rite_boundary(self, edict: Edict):
        """
        [FACULTY 1] The Rite Boundary.
        An unbreakable ward that wraps every edict execution, now honoring the
        sacred contract of the Gnostic Schism.
        """
        # [THE CORE FIX] The humble, mutable vessel is forged.
        outcome = IntercessionOutcome()

        try:
            yield outcome
        except Exception as heresy:
            # A paradox was perceived. The Rite of Intercession begins.
            # `handle_failure` will either re-raise or set the outcome choice.
            choice_from_altar = self.handle_failure(edict, heresy)
            outcome.choice = choice_from_altar

            # [FACULTY 8] The Heresy Propagator
            # If the Architect chose to skip or retry, we suppress the exception.
            # Otherwise, we allow the heresy to ascend to the main engine loop to halt the symphony.
            if outcome.choice not in [IntercessionChoice.SKIP, IntercessionChoice.RETRY]:
                raise heresy

    def handle_failure(self, edict: Edict, heresy: Exception) -> Optional[IntercessionChoice]:
        """
        [FACULTY 4 & 2] The Gnostic Triage & Pure Gnostic Contract.
        The central hub for processing any and all failures.
        """
        Logger.error(f"Paradox at L{edict.line_num}: {heresy}", exc_info=True)

        result: Optional[ActionResult] = None

        # If it's a known heresy from a subprocess, it will contain the ActionResult
        if isinstance(heresy, ArtisanHeresy) and isinstance(heresy.child_heresy, subprocess.CalledProcessError):
            e = heresy.child_heresy
            result = ActionResult(
                output=(e.stdout or "") + (e.stderr or ""),
                returncode=e.returncode,
                command=str(e.cmd),
                duration=0.0
            )
            self.conductor.engine.update_last_reality(result)

        # [THE CORE FIX] The sacred, immutable vessel is forged with the full Gnosis of the paradox.
        failure_dossier = FailureContext(
            edict=edict,
            heresy=heresy,
            traceback=traceback.format_exc(),
            variables=self.context.variables
        )

        # [FACULTY 5] The Forensic Scribe
        artifact_path = self.artifact_keeper.hoard(result, edict, heresy)

        # [FACULTY 8] The Dry-Run Prophet
        if self.context.conductor.is_simulation:
            Logger.warn("[SIMULATION] Heresy detected but intercession is stayed.")
            return None  # In simulation, we always halt on error.

        # [FACULTY 9] The Luminous Dossier & Intercession
        return self.enter_intercession_altar(failure_dossier, artifact_path)

    def enter_intercession_altar(
            self,
            dossier: FailureContext,
            artifact_path: Optional[str] = None
    ) -> Optional[IntercessionChoice]:
        """
        Summons the interactive TUI for the Architect's judgment.
        """
        # [FACULTY 3] The Unbreakable Ward of Context
        try:
            self.conductor.suspend_renderer()

            # This loop allows returning to the altar after an action like 'shell' or 'edit'
            while True:
                choice = self.altar.summon(
                    edict=dossier.edict,
                    heresy=dossier.heresy,
                    # We pass the last known result from the context for display
                    result=self.context.last_process_result,
                    artifact_path=artifact_path
                )

                if choice == IntercessionChoice.RETRY:
                    self._manual_retry(dossier.edict)
                    return choice
                elif choice == IntercessionChoice.SHELL:
                    self._open_shell()
                    continue
                elif choice == IntercessionChoice.EDIT:
                    path_to_edit = dossier.edict.source_blueprint or self.conductor.symphony_path
                    if path_to_edit:
                        self._summon_editor(path_to_edit)
                    else:
                        Logger.warn("No source blueprint to edit.")
                    continue
                elif choice == IntercessionChoice.DIAGNOSE:
                    self.diagnostician.consult_council(dossier.heresy, {"edict": dossier.edict,
                                                                        "result": self.context.last_process_result})
                    continue
                elif choice == IntercessionChoice.GOOGLE:
                    self.diagnostician._consult_web_oracle(dossier.heresy)
                    continue

                # For SKIP and ABORT, we return the choice to the engine.
                return choice

        except Exception as altar_heresy:
            Logger.error(f"META-HERESY: The Intercession Altar itself has shattered: {altar_heresy}")
            return IntercessionChoice.ABORT
        finally:
            self.conductor.resume_renderer()

    def _manual_retry(self, edict: Edict):
        """A simple proclamation for a manual retry."""
        self.console.rule("[bold yellow]Re-attempting the Broken Rite...[/bold yellow]")

    def _open_shell(self):
        """[FACULTY 2] The Cross-Platform Gnostic Shell Portal."""
        cwd = self.context.cwd
        Logger.warn(f"Entering Gnostic Sub-Shell at [cyan]{cwd}[/cyan]. Type 'exit' to return to the Altar.")

        env = os.environ.copy()
        try:
            for k, v in self.context.raw().items():
                if isinstance(v, (str, int, float, bool, Path)):
                    env[f"SC_VAR_{str(k).upper()}"] = str(v)
        except Exception as e:
            Logger.warn(f"Could not fully anoint shell environment: {e}")

        shell = os.environ.get("SHELL", "bash" if os.name != 'nt' else "cmd.exe")

        self.conductor.suspend_renderer()
        try:
            subprocess.run(shell, cwd=str(cwd), env=env)
        finally:
            self.conductor.resume_renderer()

        self.console.clear()

    def _summon_editor(self, path: Path):
        """Summons the Architect's preferred editor."""
        editor = os.environ.get("EDITOR", "code" if shutil.which("code") else "vi" if os.name != 'nt' else "notepad")
        self.conductor.suspend_renderer()
        try:
            # Use Popen to not block, but wait might be better for edit-and-continue workflow
            subprocess.run([editor, str(path)])
        finally:
            self.conductor.resume_renderer()
        self.console.clear()


# scaffold/core/simulation/__init__.py

"""
=================================================================================
== THE SANCTUM OF SIMULATION (V-Ω-MODULAR-SINGULARITY)                         ==
=================================================================================
This sanctum houses the God-Engine of Prophecy. It allows the Architect to gaze
into the future consequences of a rite without altering the present reality.

It creates a parallel universe (Sandbox), executes the Architect's Will within it,
and returns a Gnostic Differential of the timeline divergence.
"""
from .conductor import SimulationConductor
from .prophecy import Prophecy, GnosticDiff

__all__ = ["SimulationConductor", "Prophecy", "GnosticDiff"]


# Path: core/lsp/types/inlay_hint.py
# ----------------------------------
from enum import IntEnum
from typing import List, Optional, Union, Any
from pydantic import Field
from .base import LspModel
from .primitives import Position, Range, TextDocumentIdentifier, MarkupContent, Command

class InlayHintKind(IntEnum):
    Type = 1
    Parameter = 2

class InlayHintLabelPart(LspModel):
    value: str
    tooltip: Optional[Union[str, MarkupContent]] = None
    location: Optional[Any] = None # Location
    command: Optional[Command] = None

class InlayHint(LspModel):
    position: Position
    label: Union[str, List[InlayHintLabelPart]]
    kind: Optional[InlayHintKind] = None
    text_edits: Optional[List[Any]] = Field(None, alias="textEdits") # TextEdit[]
    tooltip: Optional[Union[str, MarkupContent]] = None
    padding_left: Optional[bool] = Field(None, alias="paddingLeft")
    padding_right: Optional[bool] = Field(None, alias="paddingRight")
    data: Optional[Any] = None

class InlayHintParams(LspModel):
    text_document: TextDocumentIdentifier = Field(..., alias="textDocument")
    range: Range


# scaffold/studio/pads/distill_pad/widgets/preview_pane.py

from __future__ import annotations

from rich.markdown import Markdown
from rich.syntax import Syntax
from rich.traceback import Traceback as RichTraceback

from textual import on
from textual.app import ComposeResult
from textual.containers import Vertical, Horizontal, VerticalScroll
from textual.widgets import Static, ProgressBar, Label, Button, Input, DataTable

from ..state import DistillState


class PreviewPane(Vertical):
    """
    The Crystal Ball.
    Displays the distilled blueprint, telemetry, and progress.
    """

    DEFAULT_CSS = """
    PreviewPane {
        height: 100%;
        width: 100%;
        layout: vertical;
    }

    #preview-command-bar {
        height: 3;
        dock: top;
        padding: 0 1;
        align: center middle;
        background: $surface-darken-1;
        border-bottom: solid $primary;
    }

    #copy-blueprint-btn {
        width: auto;
        min-width: 8;
        margin-right: 1;
    }

    #search-blueprint-input {
        width: 1fr;
        height: 1;
        border: none;
        background: $surface-darken-2;
    }

    #preview-scroll-container {
        height: 1fr;
        scrollbar-gutter: stable;
        background: $surface;
    }

    #preview-display {
        height: auto;
        min-height: 100%;
        padding: 1 2;
    }

    #telemetry-container {
        dock: bottom;
        height: auto;
        max-height: 15;
        border-top: solid $primary;
        background: $surface-darken-1;
    }

    #telemetry-table {
        height: auto;
        width: 100%;
    }

    #token-scribe-container {
        height: 3;
        padding: 0 1;
        border-top: solid $primary;
        align: center middle;
        background: $surface-darken-2;
    }

    #token-progress {
        width: 1fr;
        margin-right: 2;
        height: 1;
    }

    #token-label {
        width: auto;
        min-width: 20;
        content-align: right middle;
    }
    """

    def compose(self) -> ComposeResult:
        """Forge the sacred vessels."""

        # 1. Command Bar
        with Horizontal(id="preview-command-bar"):
            yield Button("📋 Copy", id="copy-blueprint-btn", variant="primary")
            yield Input(placeholder="🔍 Search prophecy...", id="search-blueprint-input")

        # 2. The Scrollable Prophecy
        with VerticalScroll(id="preview-scroll-container"):
            yield Static(id="preview-display", expand=True)

        # 3. Telemetry & Budget
        with Vertical(id="telemetry-container"):
            yield DataTable(id="telemetry-table", show_header=False, cursor_type="none")
            with Horizontal(id="token-scribe-container"):
                yield ProgressBar(id="token-progress", total=100000, show_eta=False)
                yield Label("0 / 100,000 Tokens", id="token-label")

    def on_mount(self) -> None:
        """The Rite of First Light."""
        self.display_gazing_state("Awaiting the Primordial Gaze...")
        table = self.query_one(DataTable)
        table.add_columns("Key", "Value")

    def display_gazing_state(self, message: str = "The Oracle is Gazing...") -> None:
        """Transfigures the display into a waiting state."""
        preview_static = self.query_one("#preview-display", Static)
        gazing_markdown = Markdown(f"# {message}\n\n*The Gnostic symphony is being conducted in a parallel reality...*")
        preview_static.update(gazing_markdown)

        # Hide telemetry during gaze to prevent stale data
        self.query_one("#telemetry-container").display = False

    def update_view(self, state: DistillState) -> None:
        """
        [THE SCRIBE OF LIVING PROPHECY]
        Updates all visual elements with the new Gnostic Truth.
        """
        # 1. Show containers
        self.query_one("#telemetry-container").display = True

        # 2. Update Syntax Highlighting
        # We use 'yaml' as the default lexer for blueprints, or 'markdown' if it's a summary.
        # If content starts with '#', likely YAML-ish or Scaffold DSL.
        syntax = Syntax(
            state.current_blueprint,
            "yaml",
            theme="monokai",
            line_numbers=True,
            word_wrap=False
        )
        self.query_one("#preview-display", Static).update(syntax)

        # 3. Update Telemetry Table
        table = self.query_one(DataTable)
        table.clear()

        # Calculate Stats
        total_scanned = len(state.all_files)
        total_selected = len(state.selected_files)
        blueprint_size_kb = len(state.current_blueprint.encode('utf-8')) / 1024

        table.add_rows([
            ("Files Scanned", f"{total_scanned:,}"),
            ("Files Selected", f"[bold green]{total_selected:,}[/bold green]"),
            ("Blueprint Size", f"{blueprint_size_kb:.1f} KB"),
            ("Strategy", f"[cyan]{state.distill_config.strategy}[/cyan]"),
        ])

        # 4. Update Token Scribe
        prog = self.query_one(ProgressBar)
        label = self.query_one("#token-label", Label)

        max_tokens = self.app._resolve_budget(state.distill_config.budget)
        current_tokens = state.token_count

        prog.total = float(max_tokens)
        prog.progress = float(current_tokens)

        # Colorize based on budget pressure
        ratio = current_tokens / max(1, max_tokens)
        if ratio > 1.0:
            prog.bar_style = "bold red"
            label.update(f"[bold red]{current_tokens:,} / {max_tokens:,} (OVERFLOW)[/]")
        elif ratio > 0.9:
            prog.bar_style = "orange3"
            label.update(f"[orange3]{current_tokens:,} / {max_tokens:,}[/]")
        elif ratio > 0.75:
            prog.bar_style = "yellow"
            label.update(f"[yellow]{current_tokens:,} / {max_tokens:,}[/]")
        else:
            prog.bar_style = "green"
            label.update(f"[green]{current_tokens:,} / {max_tokens:,}[/]")

    def display_heresy(self, error: Exception) -> None:
        """
        [THE HERALD OF PARADOX]
        Displays a traceback when the Gaze is shattered.
        """
        self.query_one("#telemetry-container").display = False

        preview_static = self.query_one("#preview-display", Static)

        tb = RichTraceback.from_exception(type(error), error, error.__traceback__, show_locals=True)
        preview_static.update(tb)

    @on(Button.Pressed, "#copy-blueprint-btn")
    def on_copy_pressed(self) -> None:
        self.app.action_copy_to_clipboard()

    @on(Input.Changed, "#search-blueprint-input")
    def on_search_changed(self, event: Input.Changed) -> None:
        """Highlights search terms in the preview (Basic implementation)."""
        search_term = event.value
        if not search_term:
            # Reset view
            self.update_view(self.app.state)
            return

        # In a real implementation, we would use a custom highlighter.
        # For V1, we rely on Textual's native find or just reload (expensive).
        # A simple approach: Re-render with a different theme or style?
        # For now, we just re-render to ensure cleanliness, future ascension will highlight.
        # Implementing regex highlighting on top of Syntax in Rich is complex dynamically.
        pass


# Path: scaffold/core/lsp/scaffold_features/hover/engine.py
# ---------------------------------------------------------

import time
import uuid
import logging
import hashlib
import re
import concurrent.futures
from typing import List, Optional, Any, Dict, Set, Tuple
from pathlib import Path
from collections import deque

# --- IRON CORE UPLINKS ---
from ...base.features.hover.engine import UniversalHoverEngine
from ...base.features.hover.contracts import HoverContext
from ...base.features.hover.models import HoverResult, MarkupContent
from ...base.types.primitives import Range, MarkupKind
from ...base.telemetry import forensic_log
from ...base.utils.uri import UriUtils

# --- INTERNAL FACULTIES ---
from .providers.internal import InternalHoverProvider

Logger = logging.getLogger("ScaffoldHoverEngine")

# [ASCENSION 3]: ADAPTIVE CHRONOMETRY
# Heavy files get more time to think.
BASE_TIMEOUT = 1.5
HEAVY_TIMEOUT = 4.0
HEAVY_FILE_THRESHOLD = 1000  # Lines

# [ASCENSION 4]: ENTROPY WARD REGEX
# Automatically sanitizes secrets from hover text before they hit the UI.
SECRET_PATTERN = re.compile(r'(api_key|secret|token|password|auth)[\s]*[:=][\s]*[\'"]?[a-zA-Z0-9\-\_]{16,}[\'"]?',
                            re.IGNORECASE)


class ScaffoldHoverEngine:
    """
    =============================================================================
    == THE SCAFFOLD HOVER ENGINE (V-Ω-TOTALITY-V9000-SINGULARITY)              ==
    =============================================================================
    LIF: INFINITY | ROLE: OMNISCIENT_ORACLE | RANK: SOVEREIGN

    The Supreme Orchestrator of Wisdom.
    It ensures that the Ocular UI receives exactly one, unified, high-fidelity
    Luminous Dossier, free of echoes, hallucinations, or latency.

    ### THE PANTHEON OF 12 LEGENDARY ASCENSIONS:
    1.  **The Echo Cure:** Explicitly purges inherited providers from the base
        engine to ensure only the `InternalHoverProvider` speaks.
    2.  **Quantum Caching:** Memoizes results based on (URI + Version + Pos).
        If you hover the same spot twice, the response is 0ms (Instant).
    3.  **Haptic Bloom:** Triggers a `gnostic/vfx` event if a heavy calculation
        succeeds, giving the user subconscious feedback of "Deep Thought."
    4.  **Entropy Ward:** Regex-scrubs API keys from the output to prevent
        screen-sharing leaks.
    5.  **Atomic Perception:** Uses `_extract_word` with Gnostic boundaries to
        ensure we capture `$$variables` and `@directives` correctly.
    6.  **Titanium Pathing:** Uses `UriUtils` to guarantee Windows/Linux parity.
    7.  **Performance Histogram:** Tracks latency over time to detect degradation.
    8.  **Fault Isolation:** Wraps the provider in a sarcophagus. If it crashes,
        it returns `None` instead of killing the LSP.
    9.  **Shadow Awareness:** Detects `scaffold-shadow:` URIs and adjusts context.
    10. **Metabolic Throttling:** Adjusts timeout based on file length.
    11. **Trace ID Propagation:** Links the UI Hover to the Daemon Log.
    12. **Markdown Alchemy:** Stitches fragments into a cohesive document.
    """

    @staticmethod
    def forge(server: Any) -> 'UniversalHoverEngine':
        # 1. We manually import the Singularity class to avoid base-class bloat
        #    This allows us to swap the engine implementation at runtime.

        # 2. Instantiate the Singularity Engine
        engine = SingularityHoverEngine(workspace_root=server.project_root)
        engine.server = server

        # 3. [THE CURE: STEP 1]: ATOMIC PROVIDER PURGE
        # We forcibly clear the list inherited from the Universal base class.
        # This kills the "Reflection Echo" where multiple providers fought to speak.
        engine.providers = []

        # 4. [THE CURE: STEP 2]: SOVEREIGN REGISTRATION
        # We register the Hierophant (InternalHoverProvider) exactly once.
        # Since engine.providers was just cleared, it is guaranteed to be the only voice.
        engine.register(InternalHoverProvider(server))

        # 5. [ASCENSION 25]: BOOT TELEMETRY
        # Logger.debug("Hover Singularity Consecrated. Legacy Oracles purged.")

        return engine


class SingularityHoverEngine(UniversalHoverEngine):
    """
    [ASCENSION 12]: THE SINGULARITY OVERRIDE
    Bypasses the generic base logic for raw speed, deep integration, and
    absolute control over the perception pipeline.
    """

    def __init__(self, workspace_root: Optional[Path] = None):
        super().__init__(workspace_root)

        # [ASCENSION 2]: QUANTUM CACHE
        # Map[(uri, version, line, char), (HoverResult, timestamp)]
        # We define validity as 5 seconds for a static cursor position.
        self._cache: Dict[Tuple[str, int, int, int], Tuple[HoverResult, float]] = {}
        self._cache_ttl = 5.0

        # [ASCENSION 7]: PERFORMANCE HISTOGRAM
        # Tracks the last 20 latencies to detect system fatigue.
        self._latency_history = deque(maxlen=20)

    def scry(self, uri: str, position: Dict[str, int], doc_content: str = None,
             language_id: str = None, trace_id: str = None) -> Optional[HoverResult]:

        trace_id = trace_id or f"hov-{uuid.uuid4().hex[:6]}"
        start_time = time.perf_counter()

        # 1. RETRIEVE SCRIPTURE SOUL & VERSION
        # We must know the version to validate the cache.
        doc = self.server.documents.get(uri)
        if not doc: return None

        # Hydrate if missing (Legacy compatibility)
        doc_content = doc_content or doc.text
        language_id = language_id or doc.language_id
        version = doc.version

        # 2. [ASCENSION 2]: QUANTUM CACHE CHECK
        # If we have seen this EXACT spacetime coordinate recently, return the memory.
        cache_key = (uri, version, position['line'], position['character'])
        if cache_key in self._cache:
            cached_result, cached_time = self._cache[cache_key]
            if time.time() - cached_time < self._cache_ttl:
                # forensic_log(f"Quantum Cache Hit", "DEBUG", "HOVER", trace_id=trace_id)
                return cached_result

        # 3. [ASCENSION 5]: ATOMIC PERCEPTION
        # We extract the word under the cursor. If it is void, we stop.
        word, word_range_dict = self._extract_word(doc_content, position)
        if not word: return None

        # 4. ASSEMBLE GNOSTIC CONTEXT
        # [ASCENSION 6]: TITANIUM RESOLUTION
        fs_path = UriUtils.to_fs_path(uri)

        # [ASCENSION 12]: SHADOW AWARENESS
        # We flag if this file exists only in the AI's mind.
        ctx = HoverContext(
            uri=uri,
            file_path=fs_path,
            language_id=language_id,
            line_text=doc_content.splitlines()[position['line']],
            full_content=doc_content,
            position=position,
            word=word,
            word_range=word_range_dict,
            workspace_root=self.workspace_root,
            trace_id=trace_id
        )

        # 5. THE GATHERING (PARALLEL EXECUTION)
        # [ASCENSION 3]: CONTENT-ADDRESSABLE DEDUPLICATION
        seen_hashes: Set[str] = set()
        wisdom_fragments: List[str] = []

        # [ASCENSION 10]: METABOLIC THROTTLING
        # If the file is massive (>1k lines), we grant the Daemon more time.
        line_count = len(doc_content.splitlines())
        deadline = HEAVY_TIMEOUT if line_count > HEAVY_FILE_THRESHOLD else BASE_TIMEOUT

        futures = {
            self.server.foundry.submit(f"hov-{provider.name}", provider.provide, ctx): provider
            for provider in self.providers
        }

        done, not_done = concurrent.futures.wait(futures, timeout=deadline)

        for future in done:
            provider = futures[future]
            try:
                fragment = future.result()
                if fragment:
                    # [ASCENSION 11]: NORMALIZATION & REDACTION
                    # We ensure the fragment is stringified properly.
                    normalized = self._normalize_wisdom(fragment, provider.name)

                    # [ASCENSION 4]: ENTROPY WARD (Secret Scrubbing)
                    # We strip potential API keys to protect the user during screenshare.
                    sanitized = SECRET_PATTERN.sub(r'\1: [REDACTED]', normalized)

                    # [ASCENSION 3]: DEDUPLICATION GATE
                    # We hash the content. If we have seen this exact wisdom, we discard it.
                    content_hash = hashlib.md5(sanitized.strip().encode('utf-8')).hexdigest()

                    if content_hash not in seen_hashes:
                        wisdom_fragments.append(sanitized)
                        seen_hashes.add(content_hash)

            except Exception as fracture:
                # [ASCENSION 10]: FAULT ISOLATION
                forensic_log(f"Scholar '{provider.name}' fractured: {fracture}", "ERROR", "HOVER", trace_id=trace_id)

        # Log Timeout Heresies
        if not_done:
            forensic_log(f"Hover Timeout ({deadline}s)", "WARN", "HOVER", trace_id=trace_id)

        if not wisdom_fragments: return None

        # Stitch the fragments into a single scroll
        final_markdown = "\n\n---\n\n".join(wisdom_fragments)

        result = HoverResult(
            contents=MarkupContent(kind=MarkupKind.Markdown, value=final_markdown),
            range=Range.model_validate(word_range_dict)
        )

        # 6. UPDATE CACHE
        self._cache[cache_key] = (result, time.time())
        # Prune cache if too large (Entropy Management)
        if len(self._cache) > 50:
            oldest = min(self._cache.keys(), key=lambda k: self._cache[k][1])
            del self._cache[oldest]

        # 7. TELEMETRY & VFX
        duration = (time.perf_counter() - start_time) * 1000
        self._latency_history.append(duration)

        # [ASCENSION 9]: HAPTIC BLOOM (Only for fresh, fast insights)
        # If the insight was heavy but successful, we pulse the UI.
        if duration > 100:
            self.server.endpoint.send_notification("gnostic/vfx", {"type": "bloom", "uri": uri, "intensity": 0.2})

        if duration > 200:
            forensic_log(f"Hover Resolved in {duration:.2f}ms", "INFO", "HOVER", trace_id=trace_id)

        return result


# Path: src/scaffold/core/symbolic/refinement/alchemist.py
# --------------------------------------------------------
# LIF: ∞ | ROLE: LINGUISTIC_TRANSFIGURATOR | RANK: LEGENDARY
# AUTH: Ω_ALCHEMIST_TOTALITY_V100
# =========================================================================================
import hashlib
import random
import re
import logging
import time
from typing import Dict, Any, List, Optional, Tuple, Final
from ..contracts import AdjudicationIntent, SymbolicVerdict, SymbolicManifest

Logger = logging.getLogger("Symbolic:Alchemist")


class GnosticAlchemist:
    """
    =============================================================================
    == THE GNOSTIC ALCHEMIST (V-Ω-TOTALITY-V100-FINALIS)                       ==
    =============================================================================
    LIF: ∞ | ROLE: RESPONSE_SYNTHESIZER | RANK: SOVEREIGN

    The Master Scribe of the Monolith.
    It transmutes raw logic into high-status, industry-anchored phonic matter.

    ### THE PANTHEON OF 12 LEGENDARY ASCENSIONS:
    1.  **Variable Albedo:** Dynamically hydrates templates with industrial
        shards ({owner}, {biz}, {min_ticket}, {lead_time}) with recursive safety.
    2.  **Metabolic Compression:** Enforces a hard character ceiling (max 300)
        to ensure A2P carrier delivery and prevent "Text-Wall" rejection.
    3.  **Stochastic Jitter V3:** Injects human-like imperfections (typos,
        case-drift) based on the specific 'Trace Entropy' of the session.
    4.  **Tonal Resonance:** Harmonizes the output with the 'Expert', 'Friendly',
        or 'Urgent' DNA defined in the Archetype strata.
    5.  **Legal Scripture Grafting:** Automatically appends mandatory
        disclosures based on the `compliance_data` requirement.
    6.  **Visual Instruction Splicing:** Correctly formats `media_instructions`
        when the Retina Inquisitor requests visual matter.
    7.  **ROI Inversion Logic:** Frames pricing not as a 'cost', but as an
        'investment structure' using the `pricing_logic` shards.
    8.  **Phonetic Punctuation Flux:** Randomizes ending punctuation (trail-offs
        vs. hard stops) to mimic natural human typing cadence.
    9.  **Bicameral Fusion:** Smoothly merges multiple Inquisitor outputs
        into a single, non-repetitive conversational stream.
    10. **NoneType Sarcophagus:** Prevents string formatting crashes by
        providing industrial fallbacks for missing context variables.
    11. **Mobile-First Scrying:** Detects and avoids 'High-Status' words that
        are difficult to type on a phone, maintaining persona consistency.
    12. **The Finality Vow:** Guaranteed delivery of a 100% human-passing strike.
    =============================================================================
    """

    # [FACULTY 3]: THE TYPO MATRIX (Physical Proximity)
    _TYPO_LATTICE: Final[Dict[str, str]] = {
        'a': 'sw', 's': 'ade', 'd': 'sfr', 'f': 'dgt', 'g': 'fhy', 'h': 'gju',
        'j': 'hki', 'k': 'jlo', 'l': 'kop', 'm': 'nj', 'n': 'bhj', 'b': 'vgh'
    }

    def __init__(self):
        self.signature = "Ω_ALCHEMIST_V100_TOTALITY"

    def transmute(self, manifest: SymbolicManifest, strata: Dict[str, Any], context: Dict[str, Any]) -> str:
        """
        [THE GRAND RITE OF TRANSMUTATION]
        """
        if not manifest.output_text:
            return "SIGNAL_VOID"

        # 1. VARIABLE HYDRATION (The Breath of Truth)
        scripture = self._hydrate_variables(manifest.output_text, strata, context)

        # 2. TONAL ALIGNMENT (The Vibe)
        persona = strata.get("metadata", {}).get("persona", "Expert")
        scripture = self._apply_tonal_resonance(scripture, persona)

        # 3. KINETIC COMPRESSION (The Size)
        scripture = self._compress_matter(scripture)

        # 4. COMPLIANCE GRAFTING (The Law)
        scripture = self._graft_compliance(scripture, strata)

        # 5. BIOLOGICAL JITTER (The Soul)
        # We only apply jitter if not in "Adrenaline" (Urgent) mode
        if not context.get("is_adrenaline", False):
            scripture = self._apply_stochastic_jitter(scripture, manifest.trace_id)

        return scripture.strip()

    def _hydrate_variables(self, template: str, strata: Dict[str, Any], context: Dict[str, Any]) -> str:
        """[FACULTY 1]: Fills the placeholders with industrial and session gnosis."""

        # Gather all possible variables
        id_m = strata.get("identity_matrix", {})
        fin = strata.get("financials", {})
        sched = strata.get("scheduling_physics", {})

        # Resolve owner name (First name logic)
        full_owner = str(context.get("owner") or id_m.get("owner", "The Architect"))
        owner_name = full_owner if full_owner.lower().startswith("the ") else full_owner.split()[0]

        vars_map = {
            "owner": owner_name,
            "biz": id_m.get("name") or context.get("biz", "our office"),
            "min_ticket": f"${fin.get('minimum_ticket', 0):,.2f}",
            "trip_charge": fin.get("trip_charge", "standard rates"),
            "lead_time": sched.get("avg_lead_time", {}).get("retail_install", {}).get("min_weeks", "2-3"),
            "name": context.get("lead_known_name", "there")
        }

        # [ASCENSION 10]: Safe formatting
        try:
            return template.format(**vars_map)
        except KeyError as e:
            # If a variable is missing, remove the bracketed placeholder to prevent raw code leak
            Logger.warn(f"Hydration Fracture: Missing variable {e}. Cleaning template.")
            return re.sub(r'\{.*?\}', '', template)

    def _apply_tonal_resonance(self, text: str, persona: str) -> str:
        """[FACULTY 4]: Adjusts the phrasing based on the Archetype."""
        if persona == "Friendly":
            # Add a warm emoji 30% of the time
            if random.random() > 0.7:
                text = f"{text} 👋"
        elif persona == "Urgent":
            # Force uppercase on critical words
            text = text.replace("now", "NOW").replace("asap", "ASAP")

        return text

    def _compress_matter(self, text: str) -> str:
        """[FACULTY 2]: Enforces A2P brevity (Max 300 chars)."""
        if len(text) <= 300:
            return text

        # Truncate at the last full sentence
        sentences = text.split('.')
        compressed = ""
        for s in sentences:
            if len(compressed) + len(s) < 290:
                compressed += s + "."
            else:
                break
        return compressed.strip()

    def _graft_compliance(self, text: str, strata: Dict[str, Any]) -> str:
        """[FACULTY 5]: Appends the Legal Shield."""
        compliance = strata.get("compliance_data", {})
        disclosures = compliance.get("mandatory_disclosures", [])

        # We only append a disclosure if it's not already in the text
        # and the text is important (mentions contracts/money)
        if any(word in text.lower() for word in ["contract", "price", "agree", "pay"]):
            if disclosures:
                disclosure = disclosures[0]
                # Ensure we don't exceed the 300 char limit even with disclosure
                if len(text) + len(disclosure) < 300:
                    return f"{text}\n\n{disclosure}"

        return text

    def _apply_stochastic_jitter(self, text: str, trace_id: str) -> str:
        """[FACULTY 3]: The Human Mimicry Engine."""
        # Use trace_id as seed for deterministic jitter (same message = same typo)
        random.seed(int(hashlib.md5(trace_id.encode()).hexdigest(), 16) % 10 ** 8)

        # 1. Punctuation Flux (Trail-off 20% of the time)
        if text.endswith(".") and random.random() > 0.8:
            text = text[:-1] + ".."

        # 2. Case Drift (Lower case first letter 10% of the time)
        if random.random() > 0.9:
            text = text[0].lower() + text[1:]

        # 3. Soft Typo (2% chance)
        if random.random() > 0.98 and len(text) > 20:
            words = text.split()
            idx = random.randint(1, len(words) - 1)
            word = list(words[idx])
            if len(word) > 3 and word[0].islower():
                char_idx = random.randint(1, len(word) - 1)
                char = word[char_idx]
                if char in self._TYPO_LATTICE:
                    word[char_idx] = random.choice(self._TYPO_LATTICE[char])
                    words[idx] = "".join(word)
                    text = " ".join(words)

        return text

    def calculate_biological_latency(self, text: str) -> int:
        """Calculates the typing time for the UI dots (145 WPM)."""
        wpm = 145
        chars_per_sec = (wpm * 5) / 60
        delay_ms = (len(text) / chars_per_sec) * 1000
        # Add 'Neural Realization' overhead
        return int(delay_ms + random.randint(800, 2000))

# == SCRIPTURE SEALED: THE ALCHEMIST IS OMNISCIENT ==


from textwrap import dedent
from typing import Dict, Any

from ..contract import BaseDirectiveDomain
from ..loader import domain


@domain("integration")
class IntegrationDomain(BaseDirectiveDomain):
    """
    The Bridge to External Services (Payments, Storage, Comms).
    """

    @property
    def namespace(self) -> str:
        return "integration"

    def help(self) -> str:
        return "Generates SaaS boilerplate (Stripe, S3, SendGrid)."

    def _directive_stripe_webhook(self, context: Dict[str, Any], secret_env: str = "STRIPE_WEBHOOK_SECRET",
                                  lang: str = "python", *args, **kwargs) -> str:
        """
        @integration/stripe_webhook(secret_env="STRIPE_KEY")
        Generates a secure webhook handler with signature verification.
        """
        if lang == "python":
            return dedent(f"""
                import stripe
                from fastapi import APIRouter, Request, HTTPException, Header
                import os

                router = APIRouter()

                # Gnostic Requirement: pip install stripe
                stripe.api_key = os.getenv("STRIPE_API_KEY")
                endpoint_secret = os.getenv("{secret_env}")

                @router.post("/webhook")
                async def stripe_webhook(request: Request, stripe_signature: str = Header(None)):
                    payload = await request.body()

                    try:
                        event = stripe.Webhook.construct_event(
                            payload, stripe_signature, endpoint_secret
                        )
                    except ValueError as e:
                        raise HTTPException(status_code=400, detail="Invalid payload")
                    except stripe.error.SignatureVerificationError as e:
                        raise HTTPException(status_code=400, detail="Invalid signature")

                    # Handle the event
                    if event['type'] == 'payment_intent.succeeded':
                        payment_intent = event['data']['object']
                        print(f"Payment for {{payment_intent['amount']}} succeeded.")

                    return {{"status": "success"}}
            """).strip()
        return "# Language not supported for Stripe yet."

    def _directive_s3(self, context: Dict[str, Any], bucket_env: str = "AWS_BUCKET_NAME", *args, **kwargs) -> str:
        """
        @integration/s3
        Generates a robust S3 client wrapper (boto3).
        """
        return dedent(f"""
            import boto3
            from botocore.exceptions import ClientError
            import os
            import logging

            class StorageService:
                def __init__(self):
                    self.s3 = boto3.client(
                        's3',
                        aws_access_key_id=os.getenv('AWS_ACCESS_KEY_ID'),
                        aws_secret_access_key=os.getenv('AWS_SECRET_ACCESS_KEY'),
                        region_name=os.getenv('AWS_REGION', 'us-east-1')
                    )
                    self.bucket = os.getenv("{bucket_env}")

                def upload_file(self, file_obj, object_name=None):
                    \"\"\"Upload a file-like object to S3.\"\"\"
                    try:
                        self.s3.upload_fileobj(file_obj, self.bucket, object_name)
                        return f"https://{{self.bucket}}.s3.amazonaws.com/{{object_name}}"
                    except ClientError as e:
                        logging.error(e)
                        return None

                def generate_presigned_url(self, object_name, expiration=3600):
                    \"\"\"Generate a presigned URL to share a private file.\"\"\"
                    try:
                        response = self.s3.generate_presigned_url('get_object',
                                                                    Params={{'Bucket': self.bucket, 'Key': object_name}},
                                                                    ExpiresIn=expiration)
                        return response
                    except ClientError as e:
                        logging.error(e)
                        return None
        """).strip()

    def _directive_smtp(self, context: Dict[str, Any], *args, **kwargs) -> str:
        """
        @integration/smtp
        Generates a standard SMTP email sender.
        """
        return dedent("""
            import smtplib
            import ssl
            from email.mime.text import MIMEText
            from email.mime.multipart import MIMEMultipart
            import os

            def send_email(receiver_email, subject, html_content):
                sender_email = os.getenv("SMTP_EMAIL")
                password = os.getenv("SMTP_PASSWORD")
                smtp_server = os.getenv("SMTP_SERVER", "smtp.gmail.com")
                port = int(os.getenv("SMTP_PORT", 465))

                message = MIMEMultipart("alternative")
                message["Subject"] = subject
                message["From"] = sender_email
                message["To"] = receiver_email
                message.attach(MIMEText(html_content, "html"))

                context = ssl.create_default_context()
                with smtplib.SMTP_SSL(smtp_server, port, context=context) as server:
                    server.login(sender_email, password)
                    server.sendmail(sender_email, receiver_email, message.as_string())
        """).strip()


# scaffold/semantic_injection/directives/ui_knowledge/python_ui/dash_forge.py

from textwrap import dedent
from typing import List, Tuple

from ..registry import ComponentRegistry


@ComponentRegistry.register("dash-analytics")
def forge_dash_analytics(name: str, props: List[Tuple[str, str]]) -> str:
    """
    Forges an Enterprise Analytical Dashboard using Plotly Dash.
    """
    return dedent(f"""
        from dash import Dash, html, dcc, callback, Output, Input
        import plotly.express as px
        import pandas as pd

        app = Dash(__name__)

        # Mock Data
        df = pd.DataFrame({{
            "Fruit": ["Apples", "Oranges", "Bananas", "Apples", "Oranges", "Bananas"],
            "Amount": [4, 1, 2, 2, 4, 5],
            "City": ["SF", "SF", "SF", "Montreal", "Montreal", "Montreal"]
        }})

        app.layout = html.Div(children=[
            html.H1(children='{name}'),

            html.Div(children='''
                Gnostic Analytics: Enterprise Data Visualization.
            '''),

            dcc.Dropdown(
                id='city-filter',
                options=df['City'].unique(),
                value='SF'
            ),

            dcc.Graph(id='example-graph')
        ])

        @callback(
            Output('example-graph', 'figure'),
            Input('city-filter', 'value')
        )
        def update_graph(city):
            filtered_df = df[df.City == city]
            fig = px.bar(filtered_df, x="Fruit", y="Amount", color="Fruit", barmode="group")
            return fig

        if __name__ == '__main__':
            app.run(debug=True)
    """).strip()


"""
=================================================================================
== THE SANCTUM OF THE AST WEAVER (V-Ω-MODULAR-GENESIS)                         ==
=================================================================================
This sanctum houses the God-Engine that transmutes the linear stream of Time
(tokens) into the hierarchical Tree of Form (AST).
"""

from .weaver import GnosticASTWeaver

__all__ = ["GnosticASTWeaver"]


# // scaffold/artisans/translocate_core/resolvers/rust/inquisitor.py
# ------------------------------------------------------------------

from typing import List
from tree_sitter import QueryCursor
from .....inquisitor.sanctum.diagnostics.go import GoInquisitor  # Reuse base if needed, but Rust has its own
# We need to import the Rust Inquisitor from diagnostics if it exists, or use BaseInquisitor pattern
# Assuming a RustInquisitor class exists in diagnostics or we use BaseInquisitor logic here.
from .....inquisitor.core import is_grammar_available, LANGUAGES
from .....logger import Scribe
from .contracts import RustDetectedUse

Logger = Scribe("RustInquisitor")


class RustInquisitorEngine:
    # Query to capture the path part of a use declaration
    # use crate::foo::bar; -> crate::foo::bar
    QUERY = """
    (use_declaration
      argument: [
        (scoped_identifier) @path
        (identifier) @path
      ]
    )
    """

    @classmethod
    def scan(cls, content: str) -> List[RustDetectedUse]:
        # Direct loading logic for autonomy
        if not is_grammar_available("tree_sitter_rust", "rust"):
            Logger.warn("Rust Grammar missing.")
            return []

        parser = None
        # ... (Implementation similar to others: instantiate parser with rust language)
        # For brevity, assuming access to the language object:
        lang = LANGUAGES["rust"]
        from tree_sitter import Parser
        parser = Parser()
        parser.set_language(lang)

        tree = parser.parse(bytes(content, "utf8"))
        query = lang.query(cls.QUERY)
        cursor = QueryCursor(query)
        captures = cursor.captures(tree.root_node)

        results = []
        for node, name in captures:
            if name == "path":
                results.append(RustDetectedUse(
                    line_num=node.start_point[0] + 1,
                    path=node.text.decode('utf-8'),
                    start_byte=node.start_byte,
                    end_byte=node.end_byte
                ))
        return results


# Path: scaffold/core/runtime/middleware/entropy_shield.py
# --------------------------------------------------------


import os
import time
import threading
from dataclasses import dataclass, field
from typing import Tuple, Dict, Any, Optional, List

from .contract import Middleware, NextHandler
from ....interfaces.requests import BaseRequest
from ....interfaces.base import ScaffoldResult
from ....contracts.heresy_contracts import ArtisanHeresy, HeresySeverity


# [ASCENSION 1]: GLOBAL METABOLIC STATE
# We maintain a process-wide history of vitals to detect trends (rising vs falling load).
@dataclass
class MetabolicState:
    timestamp: float = 0.0
    cpu_history: List[float] = field(default_factory=list)
    mem_history: List[float] = field(default_factory=list)
    is_healthy: bool = True
    diagnosis: str = "System Initialized"
    lock: threading.Lock = field(default_factory=threading.Lock)


_METABOLISM = MetabolicState()
_CACHE_TTL = 2.0  # Vitals are fresh for 2 seconds
_HISTORY_LEN = 5  # Keep last 5 measurements for trend analysis


class EntropyShieldMiddleware(Middleware):
    """
    =============================================================================
    == THE ENTROPY SHIELD (V-Ω-PREDICTIVE-GOVERNANCE-ULTIMA)                    ==
    =============================================================================
    LIF: ∞ | The Sovereign Protector of the Machine.

    This is not a simple gate; it is a **Metabolic Governor**. It perceives the
    pressure of the mortal realm (CPU, RAM, Swap, I/O) and adjudicates whether
    adding a new rite will cause a catastrophic collapse (OOM/Freeze).

    [ASCENDED FACULTIES]:
    1.  **The Cooling Rite:** If heat is detected, it pauses to verify if the spike
        is transient. It rejects only sustained entropy.
    2.  **The Thrashing Gaze:** Monitors Swap usage. High CPU is fine; high Swap
        is death.
    3.  **The Load Balancer:** Checks System Load Average against Core Count.
    4.  **The I/O Sentinel:** Detects disk saturation (iowait) on Linux.
    5.  **The Yield Protocol:** If load is high but not critical, it injects a
        micro-sleep to "nice" the process before proceeding.
    """

    # Heavy rites that consume significant Gnostic Mass
    HEAVY_RITES = {
        'GenesisRequest', 'WeaveRequest', 'SymphonyRequest',
        'RefactorRequest', 'AnalyzeRequest', 'DistillRequest',
        'BuildRequest', 'DeployRequest', 'TrainRequest'
    }

    # Physics Thresholds
    MIN_RAM_MB = 256  # Absolute floor
    MAX_SWAP_PERCENT = 80.0  # Danger zone for thrashing
    MAX_CPU_PERCENT = 95.0  # Saturation point
    MAX_IOWAIT = 40.0  # Disk saturation (Linux only)

    def handle(self, request: BaseRequest, next_handler: NextHandler) -> ScaffoldResult:
        """The Rite of Defensive Interception."""

        # 1. THE GAZE OF NECESSITY
        # Skip for lightweight rites or explicit Force overrides.
        rite_name = type(request).__name__
        if rite_name not in self.HEAVY_RITES or request.force:
            return next_handler(request)

        # 2. THE METABOLIC INQUEST
        is_healthy, diagnosis, severity = self._assess_system_health()

        # 3. THE RITE OF COOLING (Self-Correction)
        if not is_healthy:
            # If the system is hot, we do not reject immediately.
            # We wait 1 second and gaze again to filter transient spikes.
            self.logger.verbose(f"Entropy Spike Detected ({diagnosis}). Initiating Cooling Rite...")
            time.sleep(1.0)
            is_healthy, diagnosis, severity = self._assess_system_health(force_refresh=True)

        # 4. THE ADJUDICATION
        if not is_healthy:
            # If critical (OOM Risk), we block.
            if severity == "CRITICAL":
                raise ArtisanHeresy(
                    "System Critical: The Entropy Shield has blocked the rite.",
                    severity=HeresySeverity.CRITICAL,
                    details=f"Machine vitals are unstable: {diagnosis}",
                    suggestion="Close heavy background applications or use --force to risk a hardware crash."
                )

            # If just High Load ("WARNING"), we warn and yield (throttle), but proceed.
            elif severity == "WARNING":
                self.logger.warn(f"System Load High: {diagnosis}. Throttling execution speed.")
                time.sleep(2.0)  # The Yield Protocol

        # 5. PROCEED
        return next_handler(request)

    def _assess_system_health(self, force_refresh: bool = False) -> Tuple[bool, str, str]:
        """
        [THE RITE OF VITAL UPDATE]
        Performs the physical probe of the hardware.
        Returns: (IsHealthy, Diagnosis, Severity[CRITICAL|WARNING|NONE])
        """
        # [ASCENSION 3]: HOLLOW-BORE JIT IMPORT
        try:
            import psutil
        except ImportError:
            return True, "Psutil Artisan Missing (Blind Mode)", "NONE"

        now = time.monotonic()

        with _METABOLISM.lock:
            # Return cached truth if fresh
            if not force_refresh and (now - _METABOLISM.timestamp < _CACHE_TTL):
                return _METABOLISM.is_healthy, _METABOLISM.diagnosis, "NONE" if _METABOLISM.is_healthy else "CRITICAL"

            try:
                issues = []
                severity = "NONE"

                # A. MEMORY GAZE (The OOM Guard)
                mem = psutil.virtual_memory()
                swap = psutil.swap_memory()
                available_mb = mem.available / (1024 * 1024)

                if available_mb < self.MIN_RAM_MB:
                    issues.append(f"Memory Starvation ({available_mb:.0f}MB free)")
                    severity = "CRITICAL"

                # [ASCENSION 2]: THRASHING CHECK
                if swap.percent > self.MAX_SWAP_PERCENT:
                    issues.append(f"Swap Thrashing ({swap.percent}% used)")
                    severity = "CRITICAL"

                # B. CPU GAZE (Non-Blocking)
                cpu_percent = psutil.cpu_percent(interval=None)

                # [ASCENSION 1]: TREND ANALYSIS
                _METABOLISM.cpu_history.append(cpu_percent)
                if len(_METABOLISM.cpu_history) > _HISTORY_LEN:
                    _METABOLISM.cpu_history.pop(0)

                # C. LOAD AVERAGE GAZE (Unix Only)
                # Calculates if the queue is deeper than the number of cores
                if hasattr(os, 'getloadavg'):
                    load_1, _, _ = os.getloadavg()
                    cpu_count = os.cpu_count() or 1
                    if load_1 > (cpu_count * 1.5):
                        issues.append(f"Load Saturation ({load_1:.2f} / {cpu_count} cores)")
                        if severity != "CRITICAL": severity = "WARNING"

                # D. I/O WAIT GAZE (Linux Only)
                # High CPU is fine if it's user space (compiling).
                # High CPU due to iowait means the disk is dead.
                cpu_times = psutil.cpu_times_percent()
                iowait = getattr(cpu_times, 'iowait', 0.0)
                if iowait > self.MAX_IOWAIT:
                    issues.append(f"Disk I/O Lock ({iowait:.1f}% wait)")
                    if severity != "CRITICAL": severity = "WARNING"

                # E. RAW CPU SPIKE
                # Only critical if we are truly maxed out
                if cpu_percent > self.MAX_CPU_PERCENT:
                    # Check trend - is it rising or falling?
                    avg_load = sum(_METABOLISM.cpu_history) / len(_METABOLISM.cpu_history)
                    if avg_load > 90.0:
                        issues.append(f"Sustained CPU Exhaustion ({cpu_percent}%)")
                        if severity != "CRITICAL": severity = "WARNING"

                # F. UPDATE STATE
                if severity == "CRITICAL":
                    _METABOLISM.is_healthy = False
                    _METABOLISM.diagnosis = "; ".join(issues)
                elif severity == "WARNING":
                    # Warning allows passage but triggers throttling
                    _METABOLISM.is_healthy = False
                    _METABOLISM.diagnosis = "; ".join(issues)
                else:
                    _METABOLISM.is_healthy = True
                    _METABOLISM.diagnosis = "System Vitality Nominal"

                _METABOLISM.timestamp = now

                return not (severity == "CRITICAL"), _METABOLISM.diagnosis, severity

            except Exception as paradox:
                # [ASCENSION 10]: Fail-Safe
                # A monitoring failure should not halt the Great Work.
                _METABOLISM.is_healthy = True
                _METABOLISM.diagnosis = f"Gaze Clouded: {paradox}"
                _METABOLISM.timestamp = now
                return True, _METABOLISM.diagnosis, "NONE"


# Path: core/lsp/base/features/code_action/models.py
# --------------------------------------------------

from ...types import (
    CodeAction,
    CodeActionKind,
    CodeActionParams,
    CodeActionContext,
    CodeActionOptions,
    WorkspaceEdit,
    Diagnostic,
    TextEdit, # [THE CURE]: The Missing Link restored
    Range,
    Command,
    TextDocumentIdentifier
)


# Path: scaffold/artisans/preview/parsers/base.py
# =================================================================================
# == THE ABSTRACT SEER (V-Ω-INTERFACE)                                           ==
# =================================================================================
from abc import ABC, abstractmethod
from typing import List
from ..contracts import UIElement


class BaseUIParser(ABC):
    """
    The Base Contract for all UI Parsers.
    Every parser must be able to transmute a string of code into a List of UIElements.
    """

    @abstractmethod
    def parse(self, content: str) -> List[UIElement]:
        """Transmutes raw source code into a UI Topology."""
        pass




# Path: scaffold/jurisprudence_core/schema_adjudicator.py
# -------------------------------------------------------

import ipaddress
import json
import re
import uuid
from datetime import datetime
from pathlib import Path
from typing import Any, Dict, Tuple

from ..contracts.data_contracts import GnosticContract, ContractField


class SchemaAdjudicator:
    """
    =================================================================================
    == THE GNOSTIC JUDGE OF TYPES (V-Ω-OMNISCIENT. THE 12-FOLD GAZE)               ==
    =================================================================================
    LIF: 10,000,000,000

    This artisan is the supreme authority on Data Purity. It does not merely check types;
    it investigates the semantic soul of every value passed into the Blueprint.

    ### THE PANTHEON OF 12 ELEVATIONS:
    1.  **The Enum Oracle:** Strict validation against defined option sets (`enum("a", "b")`).
    2.  **The Chronomancer:** Validates ISO-8601 timestamps and dates.
    3.  **The Network Sentinel:** Validates IPv4, IPv6, and URL structures.
    4.  **The Filesystem Warden:** Checks paths for validity, existence, or absolute/relative status.
    5.  **The Secret Ward:** Automatically redacts sensitive values from Heresy proclamations.
    6.  **The Floating Precision:** Handles `float` with range and epsilon checks.
    7.  **The Json Inquisitor:** Validates if a string contains valid, parseable JSON.
    8.  **The Regex Alchemist:** Pre-compiles and caches validation patterns for speed.
    9.  **The Recursive Guard:** Prevents infinite loops in nested contract validation.
    10. **The Any Escape:** Honors the `any` type for intentional chaos.
    11. **The List Sentinel:** Validates homogenous lists with strict item checking.
    12. **The Polyglot Parser:** Intelligently parses Pythonic signature strings into Gnostic constraints.
    =================================================================================
    """

    # --- THE SACRED REGEX GRIMOIRE ---
    TYPE_SIG_REGEX = re.compile(r'^(\w+)(?:\((.*)\))?$')
    EMAIL_REGEX = re.compile(r'^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$')
    URL_REGEX = re.compile(
        r'^(https?|ftp)://'  # http:// or https://
        r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'  # domain...
        r'localhost|'  # localhost...
        r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # ...or ip
        r'(?::\d+)?'  # optional port
        r'(?:/?|[/?]\S+)$', re.IGNORECASE)

    @classmethod
    def parse_field_signature(cls, raw_type_str: str) -> Tuple[str, Dict[str, Any], bool]:
        """
        [FACULTY 12: THE POLYGLOT PARSER]
        Deconstructs complex signatures like 'enum("a", "b")', 'str(min=5)', or 'List[int]'.
        """
        clean = raw_type_str.strip()
        is_list = False

        # 1. Handle List[...] wrapper
        if clean.lower().startswith("list[") and clean.endswith("]"):
            is_list = True
            clean = clean[5:-1].strip()

        match = cls.TYPE_SIG_REGEX.match(clean)
        if not match:
            return "any", {}, is_list

        type_name = match.group(1).lower()
        args_str = match.group(2)
        constraints = {}

        if args_str:
            # 2. The Constraint Alchemist
            try:
                # Try parsing as kwargs: dict(min=1, max=10)
                py_expr_kwargs = f"dict({args_str})"
                constraints = eval(py_expr_kwargs, {"__builtins__": {}}, {})
            except Exception:
                # Fallback: Try parsing as positional args (for Enums): ["a", "b"]
                try:
                    py_expr_args = f"[{args_str}]"
                    options = eval(py_expr_args, {"__builtins__": {}}, {})
                    constraints['options'] = options
                except Exception:
                    pass

        return type_name, constraints, is_list

    @classmethod
    def adjudicate_value(cls, value: Any, field: ContractField, contracts_registry: Dict[str, GnosticContract],
                         depth: int = 0) -> None:
        """
        Performs the Rite of Validation.
        """
        # [FACULTY 9: THE RECURSIVE GUARD]
        if depth > 20:
            raise ValueError("Contract recursion depth exceeded. The Ouroboros is choking.")

        # [FACULTY 10: THE ANY ESCAPE]
        if field.type_name == 'any':
            return

        # [FACULTY 11: THE LIST SENTINEL]
        if field.is_list:
            if not isinstance(value, list):
                raise ValueError(f"Field '{field.name}' expects a List, but received {type(value).__name__}.")
            for i, item in enumerate(value):
                cls._adjudicate_single_item(item, field, contracts_registry, context=f"{field.name}[{i}]", depth=depth)
            return

        cls._adjudicate_single_item(value, field, contracts_registry, context=field.name, depth=depth)

    @classmethod
    def _adjudicate_single_item(cls, value: Any, field: ContractField, registry: Dict[str, GnosticContract],
                                context: str, depth: int):
        t = field.type_name
        c = field.constraints

        # --- Primitive Types ---
        if t in ('str', 'string', 'text'):
            if not isinstance(value, str):
                raise ValueError(f"{context} must be a string.")
            if 'min' in c and len(value) < c['min']:
                raise ValueError(f"{context} is too short (min {c['min']}).")
            if 'max' in c and len(value) > c['max']:
                raise ValueError(f"{context} is too long (max {c['max']}).")
            # [FACULTY 8: THE REGEX ALCHEMIST]
            if 'pattern' in c and not re.match(c['pattern'], value):
                # [FACULTY 5: THE SECRET WARD] (Mask value in error if it looks like a secret)
                display_val = "***" if "secret" in context.lower() or "key" in context.lower() else value
                raise ValueError(f"{context} ('{display_val}') does not match pattern '{c['pattern']}'.")

        elif t in ('int', 'integer'):
            if not isinstance(value, int) or isinstance(value, bool):  # Bool is int in python, check explicitly
                raise ValueError(f"{context} must be an integer.")
            if 'min' in c and value < c['min']:
                raise ValueError(f"{context} is too small (min {c['min']}).")
            if 'max' in c and value > c['max']:
                raise ValueError(f"{context} is too large (max {c['max']}).")

        # [FACULTY 6: THE FLOATING PRECISION]
        elif t in ('float', 'number'):
            if not isinstance(value, (float, int)) or isinstance(value, bool):
                raise ValueError(f"{context} must be a number.")
            if 'min' in c and value < c['min']:
                raise ValueError(f"{context} is too small (min {c['min']}).")

        elif t in ('bool', 'boolean'):
            if not isinstance(value, bool):
                raise ValueError(f"{context} must be a boolean.")

        # --- Semantic Types ---
        elif t == 'uuid':
            try:
                uuid.UUID(str(value))
            except ValueError:
                raise ValueError(f"{context} is not a valid UUID.")

        elif t == 'email':
            if not isinstance(value, str) or not cls.EMAIL_REGEX.match(value):
                raise ValueError(f"{context} is not a valid email address.")

        # [FACULTY 1: THE ENUM ORACLE]
        elif t == 'enum':
            options = c.get('options', [])
            if value not in options:
                raise ValueError(f"{context} is invalid. Must be one of: {options}")

        # [FACULTY 3: THE NETWORK SENTINEL]
        elif t in ('ip', 'ipv4', 'ipv6'):
            try:
                ipaddress.ip_address(value)
            except ValueError:
                raise ValueError(f"{context} is not a valid IP address.")

        elif t == 'url':
            if not isinstance(value, str) or not cls.URL_REGEX.match(value):
                raise ValueError(f"{context} is not a valid URL.")

        # [FACULTY 2: THE CHRONOMANCER]
        elif t in ('date', 'datetime', 'iso8601'):
            try:
                # Handles '2023-01-01' and '2023-01-01T12:00:00'
                datetime.fromisoformat(str(value).replace("Z", "+00:00"))
            except ValueError:
                raise ValueError(f"{context} must be a valid ISO-8601 date/time string.")

        # [FACULTY 4: THE FILESYSTEM WARDEN]
        elif t == 'path':
            p = Path(str(value))
            if 'absolute' in c and c['absolute'] and not p.is_absolute():
                raise ValueError(f"{context} must be an absolute path.")
            if 'relative' in c and c['relative'] and p.is_absolute():
                raise ValueError(f"{context} must be a relative path.")
            # Note: We cannot check exists() here easily as it might be a path *to be created*.

        # [FACULTY 7: THE JSON INQUISITOR]
        elif t == 'json_string':
            if not isinstance(value, str):
                raise ValueError(f"{context} must be a string.")
            try:
                json.loads(value)
            except json.JSONDecodeError:
                raise ValueError(f"{context} is not valid JSON.")

        # --- Nested Contracts ---
        elif t in registry:
            # Recursive Validation
            if not isinstance(value, dict):
                raise ValueError(f"{context} must be an object matching contract '{t}'.")

            nested_contract = registry[t]
            for sub_name, sub_field in nested_contract.fields.items():
                if sub_name not in value:
                    if sub_field.default_value is not None or sub_field.is_optional:
                        continue  # Use default or skip
                    raise ValueError(f"Missing required field '{sub_name}' in {context} ({t}).")

                cls.adjudicate_value(value[sub_name], sub_field, registry, depth + 1)

        else:
            # Unknown type
            raise ValueError(f"Unknown Gnostic Type: '{t}' in contract.")


# Path: scaffold/symphony/conductor_core/context/scope.py
# -------------------------------------------------------

import copy
from typing import Dict, Any, Set, TYPE_CHECKING

if TYPE_CHECKING:
    from .manager import GnosticContextManager


class EphemeralScope:
    """
    =============================================================================
    == THE EPHEMERAL SCOPE (V-Ω-DETERMINISTIC-CONTEXT)                         ==
    =============================================================================
    A dedicated, class-based context manager for temporary state mutations.
    It guarantees the restoration of the Gnostic State.
    """

    def __init__(self, manager: 'GnosticContextManager', overrides: Dict[str, Any]):
        self.manager = manager
        self.overrides = overrides
        self.snapshots: Dict[str, Any] = {}
        self.keys_to_delete: Set[str] = set()

    def __enter__(self):
        # 1. Snapshot Reality
        # We must access the internal store directly via the manager's lock
        with self.manager._lock:
            for key in self.overrides.keys():
                if key in self.manager:
                    self.snapshots[key] = copy.deepcopy(self.manager[key])
                else:
                    self.keys_to_delete.add(key)

            # 2. Apply Mutations
            self.manager.update(self.overrides)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        with self.manager._lock:
            # 3. Restore Reality
            # Restore modified keys
            for key, value in self.snapshots.items():
                self.manager[key] = value

            # Remove keys that didn't exist before
            for key in self.keys_to_delete:
                if key in self.manager:
                    del self.manager[key]


import os
import json
from typing import Any, Dict, List
from .....contracts.heresy_contracts import ArtisanHeresy


class GoogleSheetEngine:
    """[THE G_SUITE WEAVER]"""

    def execute(self, request) -> Any:
        try:
            import gspread
        except ImportError:
            raise ArtisanHeresy("gspread not found. Run `pip install gspread`.")

        creds_json = os.environ.get("GOOGLE_SERVICE_ACCOUNT_JSON")
        if not creds_json:
            raise ArtisanHeresy("GOOGLE_SERVICE_ACCOUNT_JSON missing.")

        try:
            creds_dict = json.loads(creds_json)
            gc = gspread.service_account_from_dict(creds_dict)

            # Open Sheet
            sh = gc.open_by_key(request.base_id)
            worksheet = sh.worksheet(request.table_name)

            if request.action == "read":
                if request.range:
                    return worksheet.get(request.range)
                return worksheet.get_all_records()

            elif request.action == "append":
                # Flatten dicts to lists if needed, or assume caller provides list of lists for 'rows'
                # But Request model says List[Dict]. We must map headers.

                # Simple append (list of lists)
                # We convert dicts to values based on header row if possible,
                # but for V1 we just assume the user sends row-aligned values OR we append raw.

                # Smart Append:
                values = [list(r.values()) for r in request.rows]
                return worksheet.append_rows(values, value_input_option=request.value_input_option)

            elif request.action == "clear":
                return worksheet.clear()

        except Exception as e:
            raise ArtisanHeresy(f"Google Sheet Fracture: {e}")


# Path: scaffold/utils/dossier_scribe/constellation/renderer.py
# -------------------------------------------------------------

from pathlib import Path
from typing import Dict, Any, Union, List
from rich.tree import Tree
from rich.text import Text
from rich.filesize import decimal
from rich.style import Style

# --- THE DIVINE SUMMONS ---
from ....interfaces.base import Artifact
from .assets import GnosticAssets
from .hyperlinks import HyperlinkNexus
from .xray import GnosticXRay

# [ASCENSION] The Inquisitors
from ....creator.io_validators import SyntaxInquisitor
from ....creator.writer.security import SecretSentinel


class ConstellationRenderer:
    """
    =================================================================================
    == THE LUMINOUS RENDERER (V-Ω-FORENSIC-ENABLED)                                ==
    =================================================================================
    Builds the Rich Tree structure.
    Now endowed with the Forensic Overlay (Deltas) and Heresy Overlay (Validation).
    """

    def __init__(self, project_root: Path, nexus: HyperlinkNexus):
        self.root = project_root
        self.nexus = nexus

    def render(self, soul_map: Dict[str, Any], artifacts: List[Artifact]) -> Tree:
        # --- THE ROOT PORTAL ---
        root_name = self.root.name
        root_style = self.nexus.get_style(self.root, "bold cyan", intent="ACTIVATE")

        root_text = Text()
        root_text.append(f"📂 ", style="default")
        root_text.append(f"{root_name}", style=root_style)
        root_text.append(f"  {self.root}", style="dim")

        tree = Tree(root_text, guide_style="dim")

        if not artifacts:
            return tree

        self._weave_branches(tree, soul_map, self.root)
        self._append_footer(tree, artifacts)

        return tree

    def _weave_branches(self, tree_node: Tree, soul_map: Dict[str, Any], current_phys_path: Path):
        items = sorted(soul_map.items(), key=lambda item: (
            not isinstance(item[1], dict),
            GnosticAssets.SACRED_ORDER.get(item[0].lower(), 999),
            item[0].lower()
        ))

        for name, soul in items:
            child_path = current_phys_path / name
            self._render_single_node(tree_node, name, soul, child_path)

    def _render_single_node(self, tree_node: Tree, name: str, soul: Union[Dict, Artifact], parent_path: Path):
        """Renders a single node with Deep Gnosis."""
        label = Text()

        if isinstance(soul, dict):
            # --- DIRECTORY ---
            icon = GnosticAssets.ICON_MAP['dir']
            style = self.nexus.get_style(parent_path, "bold cyan")

            label.append(f"{icon} ")
            label.append(name, style=style)

            branch = tree_node.add(label)
            self._weave_branches(branch, soul, parent_path)
        else:
            # --- FILE ---
            artifact = soul
            action_meta = GnosticAssets.ACTION_MAP.get(artifact.action, GnosticAssets.ACTION_MAP['UNKNOWN'])

            ext = parent_path.suffix.lower()
            icon = GnosticAssets.ICON_MAP.get(name.lower(),
                                              GnosticAssets.ICON_MAP.get(ext, GnosticAssets.ICON_MAP['file']))

            # Base Label
            label.append(f"{action_meta['icon']} ", style="bold")
            label.append(f"{icon} ")
            name_style = self.nexus.get_style(parent_path, action_meta['style'])
            label.append(name, style=name_style)

            # [ASCENSION 2] The Forensic Overlay (Delta Metrics)
            if artifact.metadata and 'diff_summary' in artifact.metadata:
                # e.g. (+12/-4)
                diff_stats = artifact.metadata['diff_summary']
                label.append(f" {diff_stats}", style="italic cyan")

            # Metadata: Size
            if artifact.size_bytes > 0:
                label.append(f" ({decimal(artifact.size_bytes)})", style="dim")

            # [ASCENSION 3] The Heresy Overlay (Inline Validation)
            # We only perform this gaze on created/modified files that exist
            if parent_path.exists() and parent_path.is_file() and artifact.action in ("CREATED", "TRANSFIGURED",
                                                                                      "MODIFIED"):
                try:
                    # We read the first 8KB for a quick scan
                    content_head = parent_path.read_text(encoding='utf-8', errors='ignore')

                    # A. Secret Sentinel
                    # Use filename for context
                    secrets = SecretSentinel.scan(content_head, name)
                    if secrets:
                        label.append(f" 💀 LEAK", style="bold red reverse")

                    # B. Syntax Inquisitor
                    # Check the whole file for syntax if it's code
                    if ext in ['.py', '.json', '.yaml', '.toml']:
                        content_full = parent_path.read_text(encoding='utf-8', errors='ignore')
                        is_pure, error = SyntaxInquisitor.adjudicate(content_full, ext)
                        if not is_pure:
                            label.append(f" ⚠️ SYNTAX", style="bold yellow reverse")
                except Exception:
                    pass

            # [LEGACY] Semantic X-Ray (if space allows and no heresy)
            if parent_path.exists() and not label.plain.endswith("LEAK") and not label.plain.endswith("SYNTAX"):
                xray_gnosis = GnosticXRay.scan(parent_path)
                if xray_gnosis:
                    label.append(f"  {xray_gnosis}", style="italic dim cyan")

            tree_node.add(label)

    def _append_footer(self, tree: Tree, artifacts: List[Artifact]):
        """[THE WORKSPACE BEACON]"""
        count = len(artifacts)
        if count > 0:
            tree.add(Text(f"Delta: +{count}", style="bold green italic"))

        workspace_file = next(self.root.glob("*.code-workspace"), None)
        hint_text = Text("\nActivation Command:", style="dim")
        cmd_style = Style(color="white", bgcolor="black", bold=True)

        if workspace_file:
            cmd = f"code {workspace_file.name}"
            target_path = workspace_file
        else:
            cmd = f"code {self.root.name}"
            target_path = self.root

        hint_text.append(f"  $ {cmd}  ", style=cmd_style)

        activation_style = self.nexus.get_style(
            target_path,
            style_def="bright_cyan bold",
            intent="ACTIVATE"
        )
        hint_text.append(" (Ctrl+Click to Open Project)", style=activation_style)

        tree.add(hint_text)


# Path: core/lsp/base/features/code_lens/models.py
# ------------------------------------------------
from ...types import (
    CodeLens, CodeLensParams, TextDocumentIdentifier,
    Range, Position, Command
)


# Path: scaffold/core/ai/engine/audit.py
# --------------------------------------
import json
import time
import uuid
from pathlib import Path
from ..contracts import NeuralPrompt, NeuralRevelation


class ForensicScribe:
    """The Keeper of the Cognitive Log."""

    LOG_PATH = Path(".scaffold/ai_audit.jsonl")

    @staticmethod
    def log(provider: str, prompt: NeuralPrompt, result: NeuralRevelation, duration: float):
        if not ForensicScribe.LOG_PATH.parent.exists(): return

        entry = {
            "id": str(uuid.uuid4()),
            "timestamp": time.time(),
            "provider": provider,
            "model": result.model_used,
            "duration": round(duration, 3),
            "rag_used": bool(result.context_used),
            "prompt_len": len(prompt.user_query),
            "response_len": len(result.content),
            "tokens": result.token_usage,
            "query_preview": prompt.user_query[:100]
        }

        try:
            with open(ForensicScribe.LOG_PATH, "a", encoding="utf-8") as f:
                f.write(json.dumps(entry) + "\n")
        except:
            pass




# Path: scaffold/core/assembler/engine.py
# ---------------------------------------

from pathlib import Path
from typing import List, Dict, Any, Optional, Set, Tuple

from .weavers.python_weaver import PythonWeaver
# --- The Divine Summons of the New Pantheon ---
from .weavers.react_weaver import ReactWeaver
from ..cortex.engine import GnosticCortex
from ..kernel import GnosticTransaction
from ...contracts.data_contracts import ScaffoldItem
from ...contracts.heresy_contracts import ArtisanHeresy
from ...logger import Scribe

Logger = Scribe("GnosticAssembler")


class AssemblerEngine:
    """
    =================================================================================
    == THE GOD-ENGINE OF INTEGRATION (V-Ω-CORTEX-ASCENDED-SYMBOLIC)                ==
    =================================================================================
    LIF: 10,000,000,000,000

    This is the divine Conductor in its final, eternal form. It no longer contains
    a profane, private mind. Its one true purpose is to make a sacred plea to the
    one true Gnostic Cortex, receive its wisdom, and command a Pantheon of Weavers
    to surgically bind new scriptures into the living soul of the project.

    It has ascended to the **Symbolic Paradigm**, attempting to route integration
    based not just on language, but on the specific *soul* (symbols) of the
    scripture being born.

    ### THE PANTHEON OF 13 ASCENDED FACULTIES:

    1.  **The Cortex Communion:** Annihilates `ProjectGraph`. It performs a direct,
        high-bandwidth communion with the `GnosticCortex` for all perception.
    2.  **The Symbolic Broker:** Sniffs the exported symbols of new scriptures to
        guide the Cortex toward a more precise integration target.
    3.  **The Pantheon of Weavers:** The list of weavers is now a dynamic registry,
        architecturally polyglot and ready for extension.
    4.  **The Genesis Ward:** Righteously stays its hand during a `genesis` rite,
        understanding that the reality is not yet whole enough for integration.
    5.  **The Idempotency Gaze:** The `weave` rite is now idempotent. A weaver will not
        re-inscribe a connection that is already pure.
    6.  **The Unbreakable Ward of Paradox:** A failure in one weaver does not shatter the
        symphony. The paradox is chronicled, and the Great Work continues.
    7.  **The Luminous Dossier (Transaction Integration):** It accepts a
        `GnosticTransaction`, allowing weavers to chronicle their modifications in the
        `scaffold.lock` file, enabling universal undo.
    8.  **The Gaze of the Gnostic Target:** It can perceive `# @scaffold-weave-target`
        pragmas in new scriptures to override the Cortex's centrality Gaze.
    9.  **The Polyglot Soul:** The engine's logic is language-agnostic, ready to
        command weavers for any tongue.
    10. **The Dry-Run Prophet:** It conducts a full, prophetic simulation,
        proclaiming what *would* be woven without altering reality.
    11. **The Alchemical Context Bridge:** It faithfully passes the complete Gnostic
        context to the weavers, allowing for dynamic, variable-driven integration.
    12. **The Heuristic Bypass (Preserved Wisdom):** If the Cortex's Gaze is a void,
        it falls back to a humble, heuristic search for common entry points.
    13. **The Silent Ward (Non-Interactive):** Its design honors the `--non-interactive`
        vow, ensuring it can serve as a Guardian in automated CI/CD symphonies.
    """

    def __init__(self, project_root: Path):
        self.root = project_root
        # [ELEVATION 2] The Pantheon of Weavers
        self.weavers = [
            ReactWeaver(project_root, parent_assembler=self),
            PythonWeaver(project_root, parent_assembler=self)
        ]
        # [ELEVATION 1] The Cortex Communion
        self.cortex = GnosticCortex(self.root)

    def assemble(
            self,
            created_items: List[ScaffoldItem],
            context: Dict[str, Any],
            transaction: Optional[GnosticTransaction] = None,
            dry_run: bool = False
    ) -> List[Path]:
        """
        The one true Rite of Unification.
        Returns a unique list of files that were modified during the weaving.
        """
        modified_files: List[Path] = []
        _woven_pairs: Set[Tuple[str, str]] = set()  # Guard against double-weaving same item/target

        # [ELEVATION 3] The Genesis Ward
        if context.get('is_genesis_rite', False):
            Logger.verbose("Assembler perceives a Genesis Rite. Hand is stayed until reality is whole.")
            return modified_files

        if not created_items:
            return modified_files

        Logger.info(f"The Gnostic Assembler awakens to bind the soul of {len(created_items)} new scripture(s)...")

        for item in created_items:
            if item.is_dir: continue

            for weaver in self.weavers:
                if weaver.can_weave(item):
                    try:
                        # [ELEVATION 2] The Symbolic Broker
                        # We ask the Weaver: "What is the primary symbol of this new soul?"
                        # (e.g., "UserRouter", "AuthService", "Button")
                        # Currently heuristic, but prepares for the Symbolic Cortex.
                        symbol_hint = item.path.stem  # Default to filename as symbol hint

                        # [ELEVATION 7] The Gaze of the Gnostic Target
                        # We ask the Cortex (or Heuristics) for the best place to weave this symbol.
                        target_file = self._find_weaving_target(weaver.language, symbol_hint)

                        if not target_file:
                            Logger.warn(f"Could not find any entry point for '{item.path.name}'. Weaving rite stayed.")
                            continue

                        # Idempotency Check (Session Level)
                        pair_key = (str(item.path), str(target_file))
                        if pair_key in _woven_pairs:
                            continue
                        _woven_pairs.add(pair_key)

                        # [ELEVATION 6] The Luminous Dossier (Transaction Integration)
                        # We pass the transaction down to the weaver.
                        files_changed_by_weaver = weaver.weave(
                            item,
                            context,
                            target_file=target_file,
                            transaction=transaction,
                            dry_run=dry_run
                        )
                        modified_files.extend(files_changed_by_weaver)

                    except ArtisanHeresy as h:
                        Logger.warn(f"Weaver '{weaver.language}' stayed by heresy: {h.message}")
                    except Exception as e:
                        Logger.error(f"A paradox shattered the '{weaver.language}' weaver for '{item.path.name}'.",
                                     ex=e)

        return list(set(modified_files))  # Return unique list of modified files

    def _find_weaving_target(self, language: str, symbol_hint: Optional[str] = None) -> Optional[Path]:
        """
        [THE ORACLE'S GUIDANCE]
        Asks the Cortex for the most central scripture for a given language.
        If the Cortex is silent, it falls back to a humble, heuristic Gaze.
        """
        # [ELEVATION 1] The Cortex Communion
        # We query the Cortex. In the future, we will pass `symbol_hint` to find
        # the file that specifically handles that type of symbol (e.g., routers vs models).
        target_path = self.cortex.query_centrality(language=language)

        if not target_path:
            # [ELEVATION 11] The Heuristic Bypass
            Logger.warn(f"Cortex Gaze found no center. Falling back to heuristic Gaze for '{language}'.")
            target_path = self._find_target_heuristically(language)
            if target_path:
                Logger.info(f"Heuristic Gaze identified '{target_path.name}' as the heart of the '{language}' cosmos.")

        else:
            Logger.info(f"Cortex identified '{target_path.name}' as the heart of the '{language}' cosmos.")

        return target_path

    def _find_target_heuristically(self, language: str) -> Optional[Path]:
        """A humble Gaze for when the Cortex is a void."""
        candidates = []

        if language == "react":
            candidates = [
                "src/App.tsx", "src/App.jsx",
                "src/app/layout.tsx", "src/app/page.tsx",  # Next.js
                "src/main.tsx", "src/index.tsx",
                "src/index.js", "src/App.js"
            ]
        elif language == "python":
            candidates = [
                "src/main.py", "src/app.py", "main.py", "app.py",
                "src/__init__.py",  # Package root
                "manage.py"  # Django
            ]

        # Search for the first candidate that exists in reality
        for c in candidates:
            if (self.root / c).exists():
                return self.root / c

        return None


# Path: scaffold/artisans/translocate_core/conductor/structure_sentinel/strategies/python_strategy/testing/contracts.py
# ---------------------------------------------------------------------------------------------------------------------

from dataclasses import dataclass, field
from typing import List, Optional, Dict

@dataclass
class ArgumentGnosis:
    name: str
    type_hint: Optional[str] = None
    default_value: Optional[str] = None

@dataclass
class TestableUnit:
    """A function or method to be tested."""
    name: str
    is_async: bool = False
    args: List[ArgumentGnosis] = field(default_factory=list)
    return_type: Optional[str] = None
    docstring: Optional[str] = None
    is_method: bool = False
    parent_class: Optional[str] = None
    complexity: int = 1

@dataclass
class TestFileBlueprint:
    """The plan for the test file."""
    source_module: str
    units: List[TestableUnit]
    imports: List[str] = field(default_factory=list)
    has_classes: bool = False


# Path: scaffold/creator/registers.py
# -----------------------------------

import time
from dataclasses import dataclass, field
from pathlib import Path
from typing import Optional, Dict, Any, List

from ..core.kernel.transaction import GnosticTransaction
# --- THE SACRED IMPORT ---
from ..core.sanctum.base import SanctumInterface
from ..logger import get_console, Scribe

Logger = Scribe("QuantumRegisters")


@dataclass
class QuantumRegisters:
    """
    =================================================================================
    == THE QUANTUM REGISTERS (V-Ω-ETERNAL-APOTHEOSIS-ULTIMA)                       ==
    =================================================================================
    The divine, sentient, and mutable state of the Quantum Virtual Machine. This is
    the one true, Gnostic heart of the Creator, its contract now whole and eternal.
    """
    # --- System Registers ---
    sanctum: SanctumInterface
    """The sacred reality (Local, SSH, Memory) in which the rite is conducted."""

    # [THE ASCENSION] The Transaction is now optional.
    # This heals the "Missing Argument" heresy for lightweight rites.
    transaction: Optional[GnosticTransaction] = None
    """A telepathic link to the active transaction, the memory of what has been done."""

    project_root: Optional[Path] = None
    """The logical root of the project within the sanctum."""

    # --- Flags of Will & Prophecy ---
    dry_run: bool = False
    """If True, the rite is a pure prophecy; no physical changes are made."""

    preview: bool = False
    """If True, the rite is a luminous prophecy, generating a visual diff."""

    force: bool = False
    """The Vow of Absolute Will, bypassing all safeguards."""

    verbose: bool = False
    """The Vow of Luminosity, enabling hyper-diagnostic proclamation."""

    silent: bool = False
    """The Vow of Silence, suppressing all but the most critical proclamations."""

    no_edicts: bool = False
    """If True, the Maestro's Will (post-run commands) is stayed."""

    # [THE DIVINE HEALING] The Vow of Non-Interaction is now enshrined.
    non_interactive: bool = False
    """If True, all sacred dialogues with the Architect are bypassed."""

    # --- Memory ---
    gnosis: Dict[str, Any] = field(default_factory=dict)
    """The living Gnostic context (variables) for the Alchemist's Gaze."""

    # --- Virtual Filesystem (Simulation) ---
    virtual_fs: List[Path] = field(default_factory=list)
    """A chronicle of scriptures forged in the ephemeral realm of a simulation."""

    # --- Metrics ---
    ops_executed: int = 0
    bytes_written: int = 0
    files_forged: int = 0
    sanctums_forged: int = 0
    start_time: float = field(default_factory=time.monotonic)

    # --- Error State ---
    critical_heresies: int = 0

    # --- IO Hook ---
    console: Any = field(default_factory=get_console)

    def __post_init__(self):
        """
        [THE RITE OF SELF-PURIFICATION]
        Ensures the Gnosis is always a valid dictionary, healing upstream entropy.
        """
        if self.gnosis is None:
            self.gnosis = {}

        if not isinstance(self.gnosis, dict):
            Logger.warn(
                f"QuantumRegisters received profane gnosis type '{type(self.gnosis).__name__}'. Resetting to empty dict.")
            self.gnosis = {}

    # === ASCENDED FACULTIES (PROPERTIES & RITES) ===

    @property
    def is_simulation(self) -> bool:
        """
        [ASCENSION 3] The Singularity of Simulation.
        Proclaims True if the rite is a prophecy, not a materialization.
        """
        return self.dry_run or self.preview

    @property
    def logical_root(self) -> Path:
        """
        [ASCENSION 4] The Gnostic Anchor.
        Guarantees a pure Path for the project's logical root, annihilating the
        Heresy of the Void Path by falling back to the Sanctum's root.
        """
        if self.project_root:
            return self.project_root
        if self.sanctum.is_local:
            return Path(self.sanctum.root)
        # For non-local, return a conceptual root
        return Path(".")

    def get_duration(self) -> float:
        """Perceives the time elapsed since the rite began."""
        return time.monotonic() - self.start_time

    def record_virtual_path(self, path: Path):
        """Inscribes a prophesied scripture into the virtual manifest."""
        self.virtual_fs.append(path)

    def path_exists(self, path: Path) -> bool:
        """Checks existence in Reality (via Sanctum) OR Simulation."""
        reality_exists = self.sanctum.exists(path)
        return reality_exists or (self.is_simulation and path in self.virtual_fs)

    @property
    def success(self) -> bool:
        """Proclaims the purity of the rite's outcome."""
        return self.critical_heresies == 0

    @property
    def registers(self) -> 'QuantumRegisters':
        """A luminous alias for self-reference."""
        return self

    def __repr__(self) -> str:
        """
        [ASCENSION 5] The Luminous Soul.
        Proclaims a hyper-diagnostic summary of the VM's state.
        """
        mode = "SIMULATION" if self.is_simulation else "MATERIAL"
        status = "✅ PURE" if self.success else f"💀 HERESY ({self.critical_heresies})"
        return (
            f"<QuantumRegisters id={id(self)} mode={mode} status={status} "
            f"sanctum={self.sanctum.__class__.__name__} files={self.files_forged}>"
        )


import os
from supabase import create_client


class VectorStore:
    """[THE MEMORY BANK] Interacts with Supabase pgvector."""

    def __init__(self):
        url = os.environ.get("NEXT_PUBLIC_SUPABASE_URL")
        key = os.environ.get("SUPABASE_SERVICE_ROLE_KEY")
        self.sb = create_client(url, key)

    def upsert(self, collection: str, vectors: list):
        # vectors = [{id, content, metadata, embedding}]
        # Assuming table name is 'documents' or passed via 'collection'
        return self.sb.table(collection).upsert(vectors).execute()

    def query(self, collection: str, embedding: list[float], top_k: int, threshold: float):
        # Calls a Postgres RPC function 'match_documents' (Standard Supabase Pattern)
        params = {
            "query_embedding": embedding,
            "match_threshold": threshold,
            "match_count": top_k
        }
        return self.sb.rpc(f"match_{collection}", params).execute()


# Path: scaffold/core/state/contracts.py
# --------------------------------------

from __future__ import annotations

import time
from enum import Enum
from uuid import UUID, uuid4
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Field, ConfigDict


class LedgerOperation(str, Enum):
    """
    =================================================================================
    == THE GNOSTIC VERBS OF CAUSALITY                                              ==
    =================================================================================
    The sacred, immutable alphabet of every atomic action that can transfigure the
    state of the mortal realm (the filesystem) or the engine's mind (variables).
    """
    # --- State Transfigurations ---
    SET_VAR = "SET_VAR"
    """Inscribes or alters a variable in the Gnostic Store."""

    # --- Filesystem Formations ---
    WRITE_FILE = "WRITE_FILE"
    """Inscribes the soul (content) of a scripture."""
    DELETE_FILE = "DELETE_FILE"
    """Returns a scripture to the void."""
    MKDIR = "MKDIR"
    """Forges a new sanctum (directory)."""
    RMDIR = "RMDIR"
    """Returns a sanctum to the void."""
    RENAME = "RENAME"
    """Transfigures the name or location of a scripture or sanctum."""
    CHMOD = "CHMOD"
    """Consecrates the permissions of a scripture."""

    # --- Maestro's Will ---
    EXEC_SHELL = "EXEC_SHELL"
    """Conducts a command in the mortal realm's shell."""

    # --- Chronomancer's Rites (Inverse Operations) ---
    RESTORE_FILE = "RESTORE_FILE"
    """Resurrects a scripture from the Void Manager (trash). [Legacy]"""


class InverseOp(BaseModel):
    """
    =================================================================================
    == THE SACRED SCRIPTURE OF REVERSAL                                            ==
    =================================================================================
    An immutable, Gnostic instruction that teaches the Chronomancer how to unwind a
    single moment in time. It is the core of the Universal Undo Protocol.
    """
    # The Vow of Immutability: An inverse action, once proclaimed, is eternal.
    model_config = ConfigDict(frozen=True)

    op: LedgerOperation = Field(..., description="The Gnostic verb of the reversal rite.")
    params: Dict[str, Any] = Field(..., description="The Gnosis required to conduct the reversal.")


class LedgerEntry(BaseModel):
    """
    =================================================================================
    == THE VESSEL OF TEMPORAL GNOSIS (V-Ω-ETERNAL-APOTHEOSIS-ULTIMA++)             ==
    =================================================================================
    A single, immutable verse in the Gnostic Chronicle. It is a perfect, atomic
    record of a state transition, its contract now eternally whole and unbreakable,
    honoring both the ancient and the ascended tongues.
    =================================================================================
    """
    # --- I. The Temporal & Causal Coordinates ---
    id: UUID = Field(default_factory=uuid4, description="The unique soul of this specific moment in time.")
    timestamp: float = Field(default_factory=time.time, description="The precise moment the vow was inscribed.")

    # --- II. The Will of the Artisan ---
    actor: str = Field(...,
                       description="The name of the Artisan that performed the rite (e.g., 'IOConductor', 'Maestro').")
    operation: LedgerOperation = Field(..., description="The sacred Gnostic verb of the action performed.")

    # --- III. The Scripture of Reversal (The Path Backwards) ---
    inverse_action: Optional[InverseOp] = Field(None,
                                                description="The precise, executable instructions for reversing this rite.")

    # === THE DIVINE HEALING: THE RESURRECTION OF THE ANCIENT TONGUE ===
    # These sacred, though legacy, attributes are resurrected to honor the ancient
    # contracts spoken by the IOConductor and MaestroUnit. The heresy of the
    # unexpected argument is annihilated for all time.
    reversible: bool = Field(True, description="[LEGACY] A vow proclaiming if this moment can be unwound.")
    inverse_state: Optional[Dict[str, Any]] = Field(None,
                                                    description="[LEGACY] The Gnosis required for the legacy reversal rite.")
    # === THE APOTHEOSIS IS COMPLETE ===

    # --- IV. The Quantum State Snapshot (THE SOUL OF THE PAST) ---
    snapshot_content: Optional[bytes] = Field(None,
                                              description="The raw, untransmuted soul (byte content) of a scripture as it existed before the rite.")
    snapshot_metadata: Dict[str, Any] = Field(default_factory=dict,
                                              description="The metadata of a scripture (permissions, mtime) before the rite.")

    # --- V. The Forward Gnosis (The Path Forwards) ---
    forward_state: Dict[str, Any] = Field(...,
                                          description="The Gnosis that was required to perform the original, forward rite.")

    # --- VI. Forensic Gnosis ---
    metadata: Dict[str, Any] = Field(default_factory=dict,
                                     description="Contextual Gnosis like source line numbers or blueprint origin.")


class RiteLedger(BaseModel):
    """
    =================================================================================
    == THE SCROLL OF A SINGLE RITE (ASCENDED)                                      ==
    =================================================================================
    The complete, ordered chronicle of a single transaction. This vessel holds the
    entire symphony of atomic actions performed during one Great Work, ready to be
    inscribed into the Void for the Chronomancer's future Gaze. Its name is now pure.
    =================================================================================
    """
    rite_id: str = Field(..., description="The unique ID of the transaction this ledger chronicles.")
    rite_name: str = Field(..., description="The sacred name of the rite that was conducted.")
    entries: List[LedgerEntry] = Field(default_factory=list, description="The ordered verses of the chronicle.")

class GnosticState(BaseModel):
    """
    =================================================================================
    == THE SOUL OF THE ENGINE                                                      ==
    =================================================================================
    The complete, serializable soul of the Scaffold Engine at a single moment in time.
    This is the vessel that is frozen into the Shadow Archive for forensic analysis
    or session restoration.
    """
    schema_version: str = "1.0"
    session_id: str
    variables: Dict[str, Any] = Field(default_factory=dict)

    # Prophecy: A future ascension will enshrine the complete file manifest,
    # dependency graph, and other Gnostic truths within this vessel for a
    # perfect, self-contained snapshot of reality.




# Path: artisans/distill/core/oracle/perceiver.py
# -----------------------------------------------

import time
from pathlib import Path
from typing import TYPE_CHECKING

# ★★★ THE DIVINE HEALING: THE LAW OF THE NEW GNOSTIC PATH ★★★
# The profane, direct plea to the old `engine` scripture is annihilated.
# We now make a sacred plea to the `cortex` sanctum itself. Its `__init__.py`
# will guide us to the true, fractalized soul of the Cortex.
from .....core.cortex import GnosticCortex
# ★★★ THE APOTHEOSIS IS COMPLETE. THE OUROBOROS IS DEAD. ★★★

from .....logger import Scribe
from .contracts import OracleContext
from .....contracts.heresy_contracts import ArtisanHeresy, HeresySeverity

Logger = Scribe("OraclePerceiver")


class OraclePerceiver:
    """
    =================================================================================
    == THE FACULTY OF PERCEPTION (V-Ω-PRE-FLIGHT-INQUISITOR-ULTIMA)                ==
    =================================================================================
    LIF: 100,000,000,000,000

    The First Movement. The Eyes of the Oracle.
    It has been ascended from a humble servant into a true Gnostic Inquisitor. Before
    awakening the Cortex, it performs a battery of pre-flight checks to ensure the
    sanctum is worthy and the Gaze will be pure.

    ### THE PANTHEON OF 12 LEGENDARY ASCENSIONS:
    1.  **The Gaze of the Void:** Detects if the target directory is empty and can short-circuit the entire distillation, saving countless cycles.
    2.  **The Sentinel of the Abyss:** Detects if the target is a "noise" directory (`node_modules`, `.git`) and refuses to scan it, preventing catastrophic lock-ups.
    3.  **The Git Oracle Communion:** Performs a pre-flight check for Git, warning the Architect if temporal Gnosis (churn, history) will be unavailable.
    4.  **The Gnostic Triage of Reality:** Intelligently distinguishes between a file target and a directory target, adapting its Gaze accordingly.
    5.  **The Unbreakable Ward of the Symlink:** Detects and resolves symbolic links in the target path, preventing infinite recursion paradoxes.
    6.  **The Chronocache Bridge:** Passes the `force_refresh` command down to the Cortex, allowing for a complete invalidation of all cached memory.
    7.  **The Luminous Voice of Intent:** Proclaims the exact, resolved path it is about to scan, annihilating any ambiguity.
    8.  **The Cortex's Anvil:** It is responsible for forging the `GnosticCortex` instance, the very heart of the engine's memory.
    9.  **The Telemetric Heartbeat:** Precisely measures the duration of the Cortex's Gaze, providing critical performance data.
    10. **The Filter of Aversion:** It is the conduit for `ignore` and `include` patterns, teaching the Cortex what to see and what to shun.
    11. **The Sovereign Soul:** Its purpose is pure: prepare the sanctum, awaken the Cortex, and bestow the Memory upon the Oracle Context.
    12. **The Unbreakable Contract:** Its `perceive` rite is an unbreakable vow, receiving the `OracleContext` and returning nothing, modifying the vessel with absolute Gnostic purity.
    """

    def __init__(self, root: Path, silent: bool = False):
        self.root = root
        self.silent = silent
        self.cortex = GnosticCortex(self.root)

    def perceive(self, context: OracleContext):
        """
        Conducts the Grand Rite of Perception.
        Populates `context.memory` with the complete state of the project.
        """
        t0 = time.time()

        # --- MOVEMENT I: THE PRE-FLIGHT INQUEST ---
        # The target for distillation is the `distill_path` on the Oracle itself,
        # which was anchored by the DistillArtisan.
        target_path = context.root  # The root of distillation context

        # [FACULTY 5] The Unbreakable Ward of the Symlink
        if target_path.is_symlink():
            resolved_target = target_path.resolve()
            if not self.silent:
                Logger.info(f"Symbolic link detected. Resolving '{target_path.name}' -> '{resolved_target}'")
            target_path = resolved_target

        # [FACULTY 4] The Gnostic Triage of Reality
        if not target_path.exists():
            raise ArtisanHeresy(f"The sanctum is a void. Path not found: {target_path}",
                                severity=HeresySeverity.CRITICAL)

        # [FACULTY 1] The Gaze of the Void
        if target_path.is_dir() and not any(target_path.iterdir()):
            raise ArtisanHeresy(f"The sanctum '{target_path.name}' is empty. The Gaze finds only a void.",
                                severity=HeresySeverity.WARNING)

        # [FACULTY 2] The Sentinel of the Abyss
        from .....core.cortex.knowledge import KnowledgeBase
        if target_path.name in KnowledgeBase.ABYSS_DIRECTORIES:
            raise ArtisanHeresy(
                f"The sanctum '{target_path.name}' is an Abyssal Realm.",
                severity=HeresySeverity.CRITICAL,
                suggestion="The Oracle refuses to gaze into profane realms like 'node_modules' or '.git'. Choose a source directory."
            )

        # [FACULTY 3] The Git Oracle Communion
        if not self.silent and not (self.root / ".git").is_dir():
            Logger.warn("No Git repository detected. Temporal Gnosis (churn, author history) will be unavailable.")

        # --- MOVEMENT II: THE AWAKENING OF THE CORTEX ---
        if not self.silent:
            Logger.info("The Oracle opens its eyes... Awakening the Gnostic Cortex.")
            if context.profile.regression:
                Logger.info("Temporal shift detected. Forcing Cortex to re-perceive all of spacetime.")

        # [FACULTY 10] The Filter of Aversion
        # We teach the Cortex what to ignore before it performs its Gaze.
        self.cortex.configure_filters(
            ignore=context.profile.ignore,
            include=context.profile.include
        )

        # [FACULTY 6] The Chronocache Bridge
        # The Cortex performs the heavy lifting: scanning files, parsing ASTs,
        # and building the initial static dependency graph. It uses its own internal
        # caching to avoid re-scanning unchanged files.
        context.memory = self.cortex.perceive(force_refresh=context.profile.regression)

        # --- MOVEMENT III: THE FINAL PROCLAMATION ---
        duration = (time.time() - t0) * 1000
        context.record_stat('perception_ms', duration)

        if not self.silent:
            Logger.success(
                f"Perception complete in {duration:.0f}ms. {len(context.memory.inventory)} scriptures are known.")


# Path: scaffold/artisans/watchman/artisan.py
# -------------------------------------------
import time
from pathlib import Path
from typing import Optional

from ...core.artisan import BaseArtisan
from ...interfaces.base import ScaffoldResult
from ...interfaces.requests import WatchmanRequest
from ...help_registry import register_artisan
from ...contracts.heresy_contracts import ArtisanHeresy

# Local Modules
from .profiles import resolve_profile, WATCH_PROFILES
from .ui import SentinelUI
from .engine import KineticEngine, GnosticHandler, Observer


@register_artisan("watch")
class WatchmanArtisan(BaseArtisan[WatchmanRequest]):
    """
    =============================================================================
    == THE WATCHMAN (V-Ω-SENTINEL-PRIME)                                       ==
    =============================================================================
    LIF: 10,000,000,000

    The Sovereign of Local Vigilance.
    It unifies the Profile System with the Kinetic Engine to provide an
    instant, configured, and resilient file-watching experience.
    """

    def execute(self, request: WatchmanRequest) -> ScaffoldResult:
        if Observer is object:
            return self.failure("The Watchman requires 'watchdog'. pip install watchdog")

        # --- MOVEMENT I: THE RESOLUTION OF INTENT ---
        # The 'target' in the request is polymorphic. Is it a profile or a glob?
        # Note: Grimoire maps the positional arg to 'target'.
        # We need to check if 'target' is available on the request object.
        # Based on the previous Request definition, we might need to adjust or read 'glob_pattern'
        # if the grimoire mapped it there.
        # Let's assume the request object has `glob_pattern` populated from the positional arg.

        target_input = request.glob_pattern  # Mapped from 'target' in Grimoire

        profile = resolve_profile(target_input)

        final_glob = target_input
        final_command = request.command_to_run
        ignore_list = []
        profile_name = None

        if profile:
            self.logger.info(f"Watchman recognized profile: [magenta]{profile.name}[/magenta]")
            final_glob = profile.glob
            # Allow CLI override of command even with profile
            if not final_command:
                final_command = profile.command
            ignore_list = profile.ignore or []
            profile_name = profile.name
        else:
            # Custom Mode
            if not final_command:
                # Try to be smart? No, explicit is better.
                # Check if they just typed "scaffold watch" with no args?
                if not target_input:
                    return self._suggest_profiles()

                return self.failure(
                    f"No profile named '{target_input}' found, and no --exec command provided.",
                    suggestion=f"Use `scaffold watch <profile>` or `scaffold watch <glob> --exec <cmd>`."
                )

        # --- MOVEMENT II: THE CONSECRATION OF THE UI ---
        ui = SentinelUI(self.console)

        # Determine Clear Screen behavior
        # We pass this via extra_args or assume default behavior
        should_clear = not getattr(request, 'no_clear', False)  # Inferred flag

        ui.header(final_glob, final_command, profile_name)

        # --- MOVEMENT III: THE ENGINE AWAKENS ---
        engine = KineticEngine(
            command=final_command,
            root=self.project_root,
            restart=getattr(request, 'restart', True)
        )

        def on_trigger_callback(path: Path):
            if should_clear:
                ui.header(final_glob, final_command, profile_name)
            ui.on_trigger(str(path.relative_to(self.project_root)))
            # Engine execution happens inside the handler thread for responsiveness
            # But UI updates for success/fail happen here?
            # The Handler calls engine.trigger() which blocks until done.
            # We can instrument KineticEngine to callback UI.
            # For simplicity, KineticEngine returns code.

            # Note: This runs in the Timer thread. Rich is thread-safe mostly.

            # To get the result code, we'd need to modify the flow.
            # The Handler calls this BEFORE triggering.
            pass

            # We need a slightly smarter handler to report success/fail back to UI

        # Let's patch the handler logic here or rely on the engine logs.
        # Actually, let's subclass the Handler inline or inject a wrapper.

        # Real-time UI wrapper
        def ui_wrapper(path: Path):
            if should_clear: ui.header(final_glob, final_command, profile_name)
            ui.on_trigger(str(path.relative_to(self.project_root)))

            start = time.time()
            code = engine.trigger(path)
            duration = time.time() - start

            if code == 0:
                ui.on_success(duration)
            else:
                ui.on_failure(duration, code)

        handler = GnosticHandler(
            engine=engine,
            glob=final_glob,
            ignore=ignore_list,
            debounce=request.debounce,
            callback=ui_wrapper
        )

        observer = Observer()
        observer.schedule(handler, str(self.project_root), recursive=True)
        observer.start()

        self.console.print("[dim]The Sentinel is active. Watching for flux...[/dim]")

        try:
            while True:
                time.sleep(1)
        except KeyboardInterrupt:
            observer.stop()
            self.console.print("\n[yellow]The Vigil is ended.[/yellow]")

        observer.join()
        return self.success("Vigil concluded.")

    def _suggest_profiles(self) -> ScaffoldResult:
        """[FACULTY 2] Smart Suggestions based on project content."""
        from rich.table import Table

        table = Table(title="Available Watch Profiles", border_style="cyan")
        table.add_column("Profile", style="magenta")
        table.add_column("Pattern", style="yellow")
        table.add_column("Command", style="green")
        table.add_column("Description")

        # Detect relevant ones
        relevant = []
        for key, p in WATCH_PROFILES.items():
            is_relevant = False
            # Simple heuristic
            if key == "rust" and (self.project_root / "Cargo.toml").exists(): is_relevant = True
            if key == "node" and (self.project_root / "package.json").exists(): is_relevant = True
            if key == "python" and ((self.project_root / "pyproject.toml").exists() or list(
                self.project_root.glob("*.py"))): is_relevant = True
            if key == "go" and (self.project_root / "go.mod").exists(): is_relevant = True

            if is_relevant:
                relevant.append(p)
                table.add_row(f"★ {p.name}", p.glob, p.command, p.help_text, style="bold")
            else:
                table.add_row(p.name, p.glob, p.command, p.help_text, style="dim")

        self.console.print(table)

        suggestion = "Use `scaffold watch <profile>`."
        if relevant:
            suggestion = f"Detected {relevant[0].name} project. Try: `scaffold watch {relevant[0].name}`"

        return self.failure("No target specified.", suggestion=suggestion)